
import os
import sys
import threading
from datetime import datetime

from syncplay import constants
from syncplay import utils
from syncplay.messages import getMessage, getLanguages, setLanguage, getInitialLanguage
from syncplay.players.playerFactory import PlayerFactory
from syncplay.utils import isBSD, isLinux, isMacOS, isWindows
from syncplay.utils import resourcespath, posixresourcespath, playerPathExists

from syncplay.vendor.Qt import QtCore, QtWidgets, QtGui, __binding__, IsPySide, IsPySide2, IsPySide6
from syncplay.vendor.Qt.QtCore import Qt, QSettings, QCoreApplication, QSize, QPoint, QUrl, QLine, QEventLoop, Signal
from syncplay.vendor.Qt.QtWidgets import QApplication, QLineEdit, QLabel, QCheckBox, QButtonGroup, QRadioButton, QDoubleSpinBox, QPlainTextEdit
from syncplay.vendor.Qt.QtGui import QCursor, QIcon, QImage, QDesktopServices
try:
    if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
        QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
except AttributeError:
    pass  # To ignore error "Attribute Qt::AA_EnableHighDpiScaling must be set before QCoreApplication is created"
if hasattr(QtCore.Qt, 'AA_UseHighDpiPixmaps'):
    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)
if IsPySide6:
    from PySide6.QtCore import QStandardPaths
elif IsPySide2:
    from PySide2.QtCore import QStandardPaths


class GuiConfiguration:
    def __init__(self, config, error=None, defaultConfig=None):
        self.defaultConfig = defaultConfig
        self.config = config
        self._availablePlayerPaths = []
        self.error = error
        constants.DEBUG_MODE = config['debug']

    def run(self):
        if QCoreApplication.instance() is None:
            self.app = QtWidgets.QApplication(sys.argv)
        dialog = ConfigDialog(self.config, self._availablePlayerPaths, self.error, self.defaultConfig)
        configLoop = QEventLoop()
        dialog.show()
        dialog.closed.connect(configLoop.quit)
        configLoop.exec_()

    def setAvailablePaths(self, paths):
        self._availablePlayerPaths = paths

    def getProcessedConfiguration(self):
        return self.config

    class WindowClosed(Exception):
        pass


class GetPlayerIconThread(threading.Thread, QtCore.QObject):
    daemon = True
    done = QtCore.Signal(str, str)

    def __init__(self):
        threading.Thread.__init__(self, name='GetPlayerIcon')
        QtCore.QObject.__init__(self)
        self.condvar = threading.Condition()
        self.playerpath = None

    def setPlayerPath(self, playerpath):
        self.condvar.acquire()
        was_none = self.playerpath is None
        self.playerpath = playerpath
        if was_none:
            self.condvar.notify()
        self.condvar.release()

    def run(self):
        while True:
            self.condvar.acquire()
            if self.playerpath is None:
                self.condvar.wait()
            playerpath = self.playerpath
            self.playerpath = None
            self.condvar.release()

            self.done.emit('spinner.mng', '')
            iconpath = PlayerFactory().getPlayerIconByPath(playerpath)
            self.done.emit(iconpath, playerpath)


class ConfigDialog(QtWidgets.QDialog):

    pressedclosebutton = True
    moreToggling = False

    closed = Signal()

    def automaticUpdatePromptCheck(self):
        if self.automaticupdatesCheckbox.checkState() == Qt.PartiallyChecked:
            reply = QtWidgets.QMessageBox.question(
                self, "Syncplay",
                getMessage("promptforupdate-label"),
                QtWidgets.QMessageBox.StandardButton.Yes | QtWidgets.QMessageBox.StandardButton.No)
            if reply == QtWidgets.QMessageBox.Yes:
                self.automaticupdatesCheckbox.setChecked(True)
            else:
                self.automaticupdatesCheckbox.setChecked(False)

    def moreToggled(self):
        self.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        if self.moreToggling is False:
            self.moreToggling = True

            if self.showmoreCheckbox.isChecked():
                self.tabListFrame.show()
                self.resetButton.show()
                self.playerargsTextbox.show()
                self.playerargsLabel.show()
                self.mediapathTextbox.show()
                self.mediapathLabel.show()
                self.mediabrowseButton.show()
                self.runButton.show()
                self.saveMoreState(True)
                self.tabListWidget.setCurrentRow(0)
                self.ensureTabListIsVisible()
                if isMacOS(): self.mediaplayerSettingsGroup.setFixedHeight(self.mediaplayerSettingsGroup.minimumSizeHint().height())
                self.stackedFrame.setFixedHeight(self.stackedFrame.minimumSizeHint().height())
            else:
                self.tabListFrame.hide()
                self.resetButton.hide()
                self.playerargsTextbox.hide()
                self.playerargsLabel.hide()
                self.runButton.hide()
                if self.mediapathTextbox.text() == "":
                    self.mediapathTextbox.hide()
                    self.mediapathLabel.hide()
                    self.mediabrowseButton.hide()
                else:
                    self.mediapathTextbox.show()
                    self.mediapathLabel.show()
                    self.mediabrowseButton.show()
                self.saveMoreState(False)
                self.stackedLayout.setCurrentIndex(0)
                if isMacOS():
                    self.mediaplayerSettingsGroup.setFixedHeight(self.mediaplayerSettingsGroup.minimumSizeHint().height())
                    newHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+50
                else:
                    newHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+13
                if self.error:
                    newHeight += self.errorLabel.height()+3
                self.stackedFrame.setFixedHeight(newHeight)
            self.adjustSize()
            if isMacOS():
                newHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+50+16
                self.setFixedWidth(self.sizeHint().width())
                self.setFixedHeight(newHeight)
            else:
                self.setFixedSize(self.sizeHint())
        self.moreToggling = False
        self.setFixedWidth(self.minimumSizeHint().width())

    def openHelp(self):
        self.QtGui.QDesktopServices.openUrl(QUrl("https://syncplay.pl/guide/client/"))

    def openRoomsDialog(self):
        RoomsDialog = QtWidgets.QDialog()
        RoomsLayout = QtWidgets.QGridLayout()
        RoomsTextbox = QtWidgets.QPlainTextEdit()
        RoomsDialog.setWindowTitle(getMessage("roomlist-msgbox-label"))
        RoomsPlaylistLabel = QtWidgets.QLabel(getMessage("roomlist-msgbox-label"))
        RoomsTextbox.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        RoomsTextbox.setPlainText(utils.getListAsMultilineString(self.config['roomList']))
        RoomsLayout.addWidget(RoomsPlaylistLabel, 0, 0, 1, 1)
        RoomsLayout.addWidget(RoomsTextbox, 1, 0, 1, 1)
        RoomsButtonBox = QtWidgets.QDialogButtonBox()
        RoomsButtonBox.setOrientation(Qt.Horizontal)
        RoomsButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
        RoomsButtonBox.accepted.connect(RoomsDialog.accept)
        RoomsButtonBox.rejected.connect(RoomsDialog.reject)
        RoomsLayout.addWidget(RoomsButtonBox, 2, 0, 1, 1)
        RoomsDialog.setLayout(RoomsLayout)
        RoomsDialog.setModal(True)
        RoomsDialog.setWindowFlags(RoomsDialog.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        RoomsDialog.show()
        result = RoomsDialog.exec_()
        if result == QtWidgets.QDialog.Accepted:
            newRooms = utils.convertMultilineStringToList(RoomsTextbox.toPlainText())
            newRooms = sorted(newRooms)
            self.relistRoomList(newRooms)

    def safenormcaseandpath(self, path):
        if utils.isURL(path):
            return path
        else:
            return os.path.normcase(os.path.normpath(path))

    def _tryToFillPlayerPath(self, playerpath, playerpathlist):
        settings = QSettings("Syncplay", "PlayerList")
        settings.beginGroup("PlayerList")
        savedPlayers = settings.value("PlayerList", [])
        if not isinstance(savedPlayers, list):
            savedPlayers = []
        else:
            for i, savedPlayer in enumerate(savedPlayers):
                savedPlayers[i] = self.safenormcaseandpath(savedPlayer)
        playerpathlist = list(set(playerpathlist + savedPlayers))
        settings.endGroup()
        foundpath = ""

        if playerpath is not None and playerpath != "":
            if utils.isURL(playerpath):
                foundpath = playerpath
                self.executablepathCombobox.addItem(foundpath)

            else:
                if not playerPathExists(playerpath):
                    expandedpath = PlayerFactory().getExpandedPlayerPathByPath(playerpath)
                    if expandedpath is not None and playerPathExists(expandedpath):
                        playerpath = expandedpath
                    elif "mpvnet.exe" in playerpath and playerPathExists(playerpath.replace("mpvnet.exe","mpvnet.com")):
                        self.executablepathCombobox.addItem(playerpath)

                if playerPathExists(playerpath):
                    foundpath = playerpath
                    self.executablepathCombobox.addItem(foundpath)

        for path in playerpathlist:
            if utils.isURL(path):
                if foundpath == "":
                    foundpath = path
                if path != playerpath:
                    self.executablepathCombobox.addItem(path)

            elif playerPathExists(path) and os.path.normcase(os.path.normpath(path)) != os.path.normcase(os.path.normpath(foundpath)):
                self.executablepathCombobox.addItem(path)
                if foundpath == "":
                    foundpath = path

        if foundpath != "":
            settings.beginGroup("PlayerList")
            playerpathlist.append(self.safenormcaseandpath(foundpath))
            settings.setValue("PlayerList", list(set(playerpathlist)))
            settings.endGroup()
        return foundpath

    @QtCore.Slot(str, str)
    def _updateExecutableIcon(self, iconpath, playerpath):
        if iconpath is not None and iconpath != "":
            if iconpath.endswith('.mng'):
                movie = QtGui.QMovie(resourcespath + iconpath)
                movie.setCacheMode(QtGui.QMovie.CacheMode.CacheAll)
                self.executableiconLabel.setMovie(movie)
                movie.start()
            else:
                self.executableiconImage.load(resourcespath + iconpath)
                self.executableiconLabel.setPixmap(QtGui.QPixmap.fromImage(self.executableiconImage))
        else:
            self.executableiconLabel.setPixmap(QtGui.QPixmap.fromImage(QtGui.QImage()))
        self.updatePlayerArguments(playerpath)

    def updateExecutableIcon(self):
        """
        Start getting the icon path in another thread, which will set the GUI
        icon if valid.

        This is performed outside the main thread because networked players may
        take a long time to perform their checks and hang the GUI while doing
        so.
        """
        currentplayerpath = str(self.executablepathCombobox.currentText())
        self._playerProbeThread.setPlayerPath(currentplayerpath)

    def updatePlayerArguments(self, currentplayerpath):
        argumentsForPath = utils.getPlayerArgumentsByPathAsText(self.perPlayerArgs, currentplayerpath)
        self.playerargsTextbox.blockSignals(True)
        self.playerargsTextbox.setText(argumentsForPath)
        self.playerargsTextbox.blockSignals(False)

    def changedPlayerArgs(self):
        currentplayerpath = self.executablepathCombobox.currentText()

        if currentplayerpath:
            NewPlayerArgs = utils.parseCommandLineString(self.playerargsTextbox.text()) if self.playerargsTextbox.text() else ""
            self.perPlayerArgs[self.executablepathCombobox.currentText()] = NewPlayerArgs

    def languageChanged(self):
        setLanguage(str(self.languageCombobox.itemData(self.languageCombobox.currentIndex())))
        QtWidgets.QMessageBox.information(self, "Syncplay", getMessage("language-changed-msgbox-label"))

    def browsePlayerpath(self):
        options = QtWidgets.QFileDialog.Options()
        defaultdirectory = ""
        browserfilter = "All files (*)"

        if os.name == 'nt':
            browserfilter = "Executable files (*.exe);;All files (*)"
            if "PROGRAMFILES(X86)" in os.environ:
                defaultdirectory = os.environ["ProgramFiles(x86)"]
            elif "PROGRAMFILES" in os.environ:
                defaultdirectory = os.environ["ProgramFiles"]
            elif "PROGRAMW6432" in os.environ:
                defaultdirectory = os.environ["ProgramW6432"]
        elif isLinux():
            defaultdirectory = "/usr/bin"
        elif isMacOS():
            defaultdirectory = "/Applications/"
        elif isBSD():
            defaultdirectory = "/usr/local/bin"

        fileName, filtr = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Browse for media player executable",
            defaultdirectory,
            browserfilter, "", options)
        if fileName:
            if isMacOS() and fileName.endswith('.app'):  # see GitHub issue #91
                # Mac OS X application bundles contain a Info.plist in the Contents subdirectory of the .app.
                # This plist file includes the 'CFBundleExecutable' key, which specifies the name of the
                # executable.  I would have used plistlib here, but since the version of this library in
                # py < 3.4 can't read from binary plist files it's pretty much useless.  Therefore, let's
                # play a game of "Guess my executable!"

                # Step 1: get all the executable files.  In a Mac OS X Application bundle, executables are stored
                # inside <bundle root>/Contents/MacOS.
                execPath = os.path.join(os.path.normpath(fileName), 'Contents', 'MacOS')
                execFiles = []
                for fn in os.listdir(execPath):
                    fn = os.path.join(execPath, fn)
                    if os.path.isfile(fn) and os.access(fn, os.X_OK):
                        execFiles.append(fn)

                # Step 2: figure out which file name looks like the application name
                baseAppName = os.path.basename(fileName).replace('.app', '').lower()
                foundExe = False
                for fn in execFiles:
                    baseExecName = os.path.basename(fn).lower()
                    if baseAppName == baseExecName:
                        fileName = fn
                        foundExe = True
                        break

                # Step 3: use the first executable in the list if no executable was found
                try:
                    if not foundExe:
                        fileName = execFiles[0]
                except IndexError:  # whoops, looks like this .app doesn't contain a executable file at all
                    pass

            self.executablepathCombobox.setEditText(os.path.normpath(fileName))

    def loadLastUpdateCheckDate(self):
        settings = QSettings("Syncplay", "Interface")
        settings.beginGroup("Update")
        try:
            self.lastCheckedForUpdates = settings.value("lastCheckedQt", None)
            if self.lastCheckedForUpdates:
                if self.config["lastCheckedForUpdates"] != None and self.config["lastCheckedForUpdates"] != "":
                    if self.lastCheckedForUpdates.toPython() > datetime.strptime(self.config["lastCheckedForUpdates"], "%Y-%m-%d %H:%M:%S.%f"):
                        self.config["lastCheckedForUpdates"] = self.lastCheckedForUpdates.toString("yyyy-MM-d HH:mm:ss.z")
                else:
                    self.config["lastCheckedForUpdates"] = self.lastCheckedForUpdates.toString("yyyy-MM-d HH:mm:ss.z")
        except:
            self.config["lastCheckedForUpdates"] = None

    def loadSavedPublicServerList(self):
        settings = QSettings("Syncplay", "Interface")
        settings.beginGroup("PublicServerList")
        self.publicServers = settings.value("publicServers", None)

    def loadMediaBrowseSettings(self):
        settings = QSettings("Syncplay", "MediaBrowseDialog")
        settings.beginGroup("MediaBrowseDialog")
        self.mediadirectory = settings.value("mediadir", "")
        settings.endGroup()

    def saveMediaBrowseSettings(self):
        settings = QSettings("Syncplay", "MediaBrowseDialog")
        settings.beginGroup("MediaBrowseDialog")
        settings.setValue("mediadir", self.mediadirectory)
        settings.endGroup()

    def getMoreState(self):
        settings = QSettings("Syncplay", "MoreSettings")
        settings.beginGroup("MoreSettings")
        morestate = str.lower(str(settings.value("ShowMoreSettings", "false")))
        settings.endGroup()
        if morestate == "true":
            return True
        else:
            return False

    def saveMoreState(self, morestate):
        settings = QSettings("Syncplay", "MoreSettings")
        settings.beginGroup("MoreSettings")
        settings.setValue("ShowMoreSettings", morestate)
        settings.endGroup()

    def updateServerList(self):
        try:
            servers = utils.getListOfPublicServers()
        except IOError as e:
            self.showErrorMessage(e.args[0])
            return
        currentServer = self.hostCombobox.currentText()
        self.hostCombobox.clear()
        if servers:
            i = 0
            for server in servers:
                self.hostCombobox.addItem(server[1])
                self.hostCombobox.setItemData(i, server[0], Qt.ToolTipRole)
                i += 1
            settings = QSettings("Syncplay", "Interface")
            settings.beginGroup("PublicServerList")
            settings.setValue("publicServers", servers)
        self.hostCombobox.setEditText(currentServer)

    def fillRoomsCombobox(self):
        previousRoomSelection = self.roomsCombobox.currentText()
        self.roomsCombobox.clear()
        for roomListValue in self.config['roomList']:
            self.roomsCombobox.addItem(roomListValue)
        self.roomsCombobox.setEditText(previousRoomSelection)

    def relistRoomList(self, newRooms):
        filteredNewRooms = [room for room in newRooms if room and not room.isspace()]
        self.config['roomList'] = filteredNewRooms
        self.fillRoomsCombobox()

    def addRoomToList(self, newRoom=None):
        if newRoom is None:
            newRoom = self.roomsCombobox.currentText()
        if not newRoom:
            return
        roomList = self.config['roomList']
        if newRoom not in roomList:
            roomList.append(newRoom)
        roomList = sorted(roomList)
        self.config['roomList'] = roomList

    def showErrorMessage(self, errorMessage):
        QtWidgets.QMessageBox.warning(self, "Syncplay", errorMessage)

    def browseMediapath(self):
        self.loadMediaBrowseSettings()
        options = QtWidgets.QFileDialog.Options()
        if IsPySide:
            if self.config["mediaSearchDirectories"] and os.path.isdir(self.config["mediaSearchDirectories"][0]):
                defaultdirectory = self.config["mediaSearchDirectories"][0]
            elif os.path.isdir(self.mediadirectory):
                defaultdirectory = self.mediadirectory
            elif os.path.isdir(QDesktopServices.storageLocation(QDesktopServices.MoviesLocation)):
                defaultdirectory = QDesktopServices.storageLocation(QDesktopServices.MoviesLocation)
            elif os.path.isdir(QDesktopServices.storageLocation(QDesktopServices.HomeLocation)):
                defaultdirectory = QDesktopServices.storageLocation(QDesktopServices.HomeLocation)
            else:
                defaultdirectory = ""
        elif IsPySide6 or IsPySide2:
            if self.config["mediaSearchDirectories"] and os.path.isdir(self.config["mediaSearchDirectories"][0]):
                defaultdirectory = self.config["mediaSearchDirectories"][0]
            elif os.path.isdir(self.mediadirectory):
                defaultdirectory = self.mediadirectory
            elif os.path.isdir(QStandardPaths.standardLocations(QStandardPaths.MoviesLocation)[0]):
                defaultdirectory = QStandardPaths.standardLocations(QStandardPaths.MoviesLocation)[0]
            elif os.path.isdir(QStandardPaths.standardLocations(QStandardPaths.HomeLocation)[0]):
                defaultdirectory = QStandardPaths.standardLocations(QStandardPaths.HomeLocation)[0]
            else:
                defaultdirectory = ""
        browserfilter = "All files (*)"
        fileName, filtr = QtWidgets.QFileDialog.getOpenFileName(
            self, "Browse for media files", defaultdirectory,
            browserfilter, "", options)
        if fileName:
            self.mediapathTextbox.setText(os.path.normpath(fileName))
            self.mediadirectory = os.path.dirname(fileName)
            self.saveMediaBrowseSettings()

    def _runWithoutStoringConfig(self):
        self._saveDataAndLeave(False)

    def _saveDataAndLeave(self, storeConfiguration=True):
        self.config['noStore'] = not storeConfiguration
        if storeConfiguration:
            self.automaticUpdatePromptCheck()
        self.loadLastUpdateCheckDate()

        self.config["perPlayerArguments"] = self.perPlayerArgs
        self.config["mediaSearchDirectories"] = utils.convertMultilineStringToList(self.mediasearchTextEdit.toPlainText())
        self.config["trustedDomains"] = utils.convertMultilineStringToList(self.trusteddomainsTextEdit.toPlainText())

        if self.serverpassTextbox.isEnabled():
            self.config['password'] = self.serverpassTextbox.text()
        self.processWidget(self, lambda w: self.saveValues(w))
        if self.hostCombobox.currentText():
            self.config['host'] = self.hostCombobox.currentText() if ":" in self.hostCombobox.currentText() else self.hostCombobox.currentText() + ":" + str(constants.DEFAULT_PORT)
            self.config['host'] = self.config['host'].replace(" ", "").replace("\t", "").replace("\n", "").replace("\r", "")
        else:
            self.config['host'] = None
        self.config['playerPath'] = str(self.safenormcaseandpath(self.executablepathCombobox.currentText()))
        self.config['language'] = str(self.languageCombobox.itemData(self.languageCombobox.currentIndex()))
        if self.mediapathTextbox.text() == "":
            self.config['file'] = None
        elif os.path.isfile(os.path.abspath(self.mediapathTextbox.text())):
            self.config['file'] = os.path.abspath(self.mediapathTextbox.text())
        else:
            self.config['file'] = str(self.mediapathTextbox.text())
        self.config['publicServers'] = self.publicServerAddresses
        self.config['room'] = self.roomsCombobox.currentText()
        if self.config['autosaveJoinsToList']:
            self.addRoomToList(self.config['room'])

        self.pressedclosebutton = False
        self.close()
        self.closed.emit()

    def closeEvent(self, event):
        if self.pressedclosebutton:
            super(ConfigDialog, self).closeEvent(event)
            self.closed.emit()
            sys.exit()

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            sys.exit()

    def dragEnterEvent(self, event):
        data = event.mimeData()
        urls = data.urls()
        if urls and urls[0].scheme() == 'file':
            event.acceptProposedAction()

    def dropEvent(self, event):
        data = event.mimeData()
        urls = data.urls()
        if urls and urls[0].scheme() == 'file':
            dropfilepath = os.path.abspath(str(event.mimeData().urls()[0].toLocalFile()))
            if dropfilepath[-4:].lower() == ".exe":
                self.executablepathCombobox.setEditText(dropfilepath)
            else:
                self.mediapathTextbox.setText(dropfilepath)
                self.mediapathTextbox.show()
                self.mediapathLabel.show()
                self.mediabrowseButton.show()
                if not self.showmoreCheckbox.isChecked():
                    newHeight = self.connectionSettingsGroup.minimumSizeHint().height() + self.mediaplayerSettingsGroup.minimumSizeHint().height() + self.bottomButtonFrame.minimumSizeHint().height() + 3
                    if self.error:
                        newHeight += self.errorLabel.height() + 3
                    self.stackedFrame.setFixedHeight(newHeight)
                    self.adjustSize()
                    self.setFixedSize(self.sizeHint())

    def processWidget(self, container, torun):
        for widget in container.children():
            self.processWidget(widget, torun)
            if hasattr(widget, 'objectName') and widget.objectName() and widget.objectName()[:3] != "qt_":
                torun(widget)

    def loadTooltips(self, widget):
        tooltipName = widget.objectName().lower().split(constants.CONFIG_NAME_MARKER)[0] + "-tooltip"
        if tooltipName[:1] == constants.INVERTED_STATE_MARKER or tooltipName[:1] == constants.LOAD_SAVE_MANUALLY_MARKER:
            tooltipName = tooltipName[1:]
        widget.setToolTip(getMessage(tooltipName))

    def loadValues(self, widget):
        valueName = str(widget.objectName())
        if valueName[:1] == constants.LOAD_SAVE_MANUALLY_MARKER:
            return

        if isinstance(widget, QCheckBox) and widget.objectName():
            if valueName[:1] == constants.INVERTED_STATE_MARKER:
                valueName = valueName[1:]
                inverted = True
            else:
                inverted = False
            if self.config[valueName] is None:
                widget.setTristate(True)
                widget.setCheckState(Qt.PartiallyChecked)
                widget.stateChanged.connect(lambda: widget.setTristate(False))
            else:
                widget.setChecked(self.config[valueName] != inverted)
        elif isinstance(widget, QRadioButton):
            radioName, radioValue = valueName.split(constants.CONFIG_NAME_MARKER)[1].split(constants.CONFIG_VALUE_MARKER)
            if self.config[radioName] == radioValue:
                widget.setChecked(True)
        elif isinstance(widget, QLineEdit):
            widget.setText(self.config[valueName])

    def saveValues(self, widget):
        valueName = str(widget.objectName())
        if valueName[:1] == constants.LOAD_SAVE_MANUALLY_MARKER:
            return

        if isinstance(widget, QCheckBox) and widget.objectName():
            if widget.checkState() == Qt.PartiallyChecked:
                self.config[valueName] = None
            else:
                if valueName[:1] == constants.INVERTED_STATE_MARKER:
                    valueName = valueName[1:]
                    inverted = True
                else:
                    inverted = False
                self.config[valueName] = widget.isChecked() != inverted
        elif isinstance(widget, QRadioButton):
            radioName, radioValue = valueName.split(constants.CONFIG_NAME_MARKER)[1].split(constants.CONFIG_VALUE_MARKER)
            if widget.isChecked():
                self.config[radioName] = radioValue
        elif isinstance(widget, QLineEdit):
            self.config[valueName] = widget.text()

    def connectChildren(self, widget):
        widgetName = str(widget.objectName())
        if widgetName in self.subitems:
            widget.stateChanged.connect(lambda: self.updateSubwidgets(self, widget))
            self.updateSubwidgets(self, widget)

    def updateSubwidgets(self, container, parentwidget, subwidgets=None):
        widgetName = parentwidget.objectName()
        if not subwidgets:
            subwidgets = self.subitems[widgetName]
        for widget in container.children():
            self.updateSubwidgets(widget, parentwidget, subwidgets)
            if hasattr(widget, 'objectName') and widget.objectName() and widget.objectName() in subwidgets:
                widget.setDisabled(not parentwidget.isChecked())

    def addBasicTab(self):
        config = self.config
        playerpaths = self.playerpaths
        error = self.error
        if self.datacleared == True:
            error = constants.ERROR_MESSAGE_MARKER + "{}".format(getMessage("gui-data-cleared-notification"))
            self.error = error
        if config['host'] is None:
            host = ""
        elif ":" in config['host'] and '[' not in config['host']:
            host = config['host']
        else:
            host = config['host'] + ":" + str(config['port'])

        self.perPlayerArgs = self.config["perPlayerArguments"]
        self.mediaSearchDirectories = self.config["mediaSearchDirectories"]
        self.trustedDomains = self.config["trustedDomains"]

        self.connectionSettingsGroup = QtWidgets.QGroupBox(getMessage("connection-group-title"))
        self.loadSavedPublicServerList()
        self.hostCombobox = QtWidgets.QComboBox(self)
        if self.publicServers:
            i = 0
            for publicServer in self.publicServers:
                serverTitle = publicServer[0]
                serverAddressPort = publicServer[1]
                self.hostCombobox.addItem(serverAddressPort)
                self.hostCombobox.setItemData(i, serverTitle, Qt.ToolTipRole)
                if serverAddressPort not in self.publicServerAddresses:
                    self.publicServerAddresses.append(serverAddressPort)
                i += 1
        self.hostCombobox.setEditable(True)
        self.hostCombobox.setEditText(host)
        self.hostLabel = QLabel(getMessage("host-label"), self)
        self.usernameTextbox = QLineEdit(self)

        self.usernameTextbox.setObjectName("name")
        self.serverpassLabel = QLabel(getMessage("password-label"), self)
        self.roomsCombobox = QtWidgets.QComboBox(self)
        self.roomsCombobox.setEditable(True)
        caseSensitiveCompleter = QtWidgets.QCompleter(self)
        caseSensitiveCompleter.setCaseSensitivity(Qt.CaseSensitive)
        self.roomsCombobox.setCompleter(caseSensitiveCompleter)
        self.fillRoomsCombobox()
        self.roomsCombobox.setEditText(config['room'])
        self.usernameLabel = QLabel(getMessage("name-label"), self)
        self.serverpassTextbox = QLineEdit(self)
        self.serverpassTextbox.setText(self.storedPassword)
        self.defaultroomLabel = QLabel(getMessage("room-label"), self)
        self.editRoomsButton = QtWidgets.QToolButton()
        self.editRoomsButton.setIcon(QtGui.QIcon(resourcespath + 'bullet_edit_centered.png'))
        self.editRoomsButton.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "edit-rooms")
        self.editRoomsButton.released.connect(self.openRoomsDialog)
        self.hostLabel.setObjectName("host")
        self.hostCombobox.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "host")
        self.usernameLabel.setObjectName("name")
        self.usernameTextbox.setObjectName("name")
        self.serverpassLabel.setObjectName("password")
        self.serverpassTextbox.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "password")
        self.hostCombobox.editTextChanged.connect(self.updatePasswordVisibilty)
        self.hostCombobox.currentIndexChanged.connect(self.updatePasswordVisibilty)
        self.defaultroomLabel.setObjectName("room")
        self.roomsCombobox.setObjectName("room")

        self.connectionSettingsLayout = QtWidgets.QGridLayout()
        self.connectionSettingsLayout.addWidget(self.hostLabel, 0, 0)
        self.connectionSettingsLayout.addWidget(self.hostCombobox, 0, 1)
        self.connectionSettingsLayout.addWidget(self.serverpassLabel, 1, 0)
        self.connectionSettingsLayout.addWidget(self.serverpassTextbox, 1, 1)
        self.connectionSettingsLayout.addWidget(self.usernameLabel, 2, 0)
        self.connectionSettingsLayout.addWidget(self.usernameTextbox, 2, 1)
        self.connectionSettingsLayout.addWidget(self.defaultroomLabel, 3, 0)
        self.connectionSettingsLayout.addWidget(self.editRoomsButton, 3, 2, Qt.AlignRight)
        self.connectionSettingsLayout.addWidget(self.roomsCombobox, 3, 1)
        self.connectionSettingsLayout.setSpacing(10)
        self.connectionSettingsGroup.setLayout(self.connectionSettingsLayout)
        if isMacOS():
            self.connectionSettingsGroup.setFixedHeight(self.connectionSettingsGroup.minimumSizeHint().height())
        else:
            self.connectionSettingsGroup.setMaximumHeight(self.connectionSettingsGroup.minimumSizeHint().height())

        self.playerargsTextbox = QLineEdit("", self)
        self.playerargsTextbox.textEdited.connect(self.changedPlayerArgs)
        self.playerargsLabel = QLabel(getMessage("player-arguments-label"), self)

        self.mediaplayerSettingsGroup = QtWidgets.QGroupBox(getMessage("media-setting-title"))
        self.executableiconImage = QtGui.QImage()
        self.executableiconLabel = QLabel(self)
        self.executableiconLabel.setFixedWidth(16)
        self.executableiconLabel.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        self.executablepathCombobox = QtWidgets.QComboBox(self)
        self.executablepathCombobox.setEditable(True)
        self.executablepathCombobox.currentIndexChanged.connect(self.updateExecutableIcon)
        self.executablepathCombobox.setEditText(self._tryToFillPlayerPath(config['playerPath'], playerpaths))
        self.executablepathCombobox.editTextChanged.connect(self.updateExecutableIcon)

        self.executablepathLabel = QLabel(getMessage("executable-path-label"), self)
        self.executablebrowseButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'folder_explore.png'), getMessage("browse-label"))
        self.executablebrowseButton.clicked.connect(self.browsePlayerpath)
        self.mediapathTextbox = QLineEdit(config['file'], self)
        self.mediapathLabel = QLabel(getMessage("media-path-label"), self)
        self.mediabrowseButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'folder_explore.png'), getMessage("browse-label"))
        self.mediabrowseButton.clicked.connect(self.browseMediapath)

        self.executablepathLabel.setObjectName("executable-path")
        self.executablepathCombobox.setObjectName("executable-path")
        self.executablepathCombobox.setMinimumContentsLength(constants.EXECUTABLE_COMBOBOX_MINIMUM_LENGTH)
        if not IsPySide6:
            self.executablepathCombobox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToMinimumContentsLength)
        else:
            self.executablepathCombobox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToMinimumContentsLengthWithIcon)
        self.mediapathLabel.setObjectName("media-path")
        self.mediapathTextbox.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "media-path")
        self.playerargsLabel.setObjectName("player-arguments")
        self.playerargsTextbox.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "player-arguments")

        self.mediaplayerSettingsLayout = QtWidgets.QGridLayout()
        self.mediaplayerSettingsLayout.addWidget(self.executablepathLabel, 0, 0, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.executableiconLabel, 0, 1, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.executablepathCombobox, 0, 2, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.executablebrowseButton, 0, 3, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.mediapathLabel, 1, 0, 1, 2)
        self.mediaplayerSettingsLayout.addWidget(self.mediapathTextbox, 1, 2, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.mediabrowseButton, 1, 3, 1, 1)
        self.mediaplayerSettingsLayout.addWidget(self.playerargsLabel, 2, 0, 1, 2)
        self.mediaplayerSettingsLayout.addWidget(self.playerargsTextbox, 2, 2, 1, 2)
        self.mediaplayerSettingsLayout.setSpacing(10)
        self.mediaplayerSettingsGroup.setLayout(self.mediaplayerSettingsLayout)

        iconWidth = self.executableiconLabel.minimumSize().width()+self.mediaplayerSettingsLayout.spacing()
        maxWidth = max(
            self.hostLabel.minimumSizeHint().width(),
            self.usernameLabel.minimumSizeHint().width(),
            self.serverpassLabel.minimumSizeHint().width(),
            self.defaultroomLabel.minimumSizeHint().width(),
            self.executablepathLabel.minimumSizeHint().width(),
            self.mediapathLabel.minimumSizeHint().width(),
            self.playerargsLabel.minimumSizeHint().width()
        )

        self.hostLabel.setMinimumWidth(maxWidth+iconWidth)
        self.usernameLabel.setMinimumWidth(maxWidth+iconWidth)
        self.serverpassLabel.setMinimumWidth(maxWidth+iconWidth)
        self.defaultroomLabel.setMinimumWidth(maxWidth+iconWidth)
        self.executablepathLabel.setMinimumWidth(maxWidth)
        self.mediapathLabel.setMinimumWidth(maxWidth+iconWidth)
        self.playerargsLabel.setMinimumWidth(maxWidth+iconWidth)

        self.showmoreCheckbox = QCheckBox(getMessage("more-title"))
        self.showmoreCheckbox.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "more")

        self.basicOptionsFrame = QtWidgets.QFrame()
        self.basicOptionsLayout = QtWidgets.QVBoxLayout()
        if error:
            error = str(error)
            self.errorLabel = QLabel(self)
            if error[:1] != constants.ERROR_MESSAGE_MARKER:
                self.errorLabel.setStyleSheet(constants.STYLE_ERRORLABEL)
            else:
                error = error[1:]
                self.errorLabel.setStyleSheet(constants.STYLE_SUCCESSLABEL)
            self.errorLabel.setText(error)
            self.errorLabel.setAlignment(Qt.AlignCenter)
            self.basicOptionsLayout.addWidget(self.errorLabel)
        self.connectionSettingsGroup.setMaximumHeight(self.connectionSettingsGroup.minimumSizeHint().height())
        self.basicOptionsLayout.setAlignment(Qt.AlignTop)
        self.basicOptionsLayout.addWidget(self.connectionSettingsGroup)
        self.basicOptionsLayout.addSpacing(5)
        self.basicOptionsLayout.addWidget(self.mediaplayerSettingsGroup)
        self.basicOptionsFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.basicOptionsFrame.setLayout(self.basicOptionsLayout)
        self.stackedLayout.addWidget(self.basicOptionsFrame)

    def addReadinessTab(self):
        self.readyFrame = QtWidgets.QFrame()
        self.readyLayout = QtWidgets.QVBoxLayout()
        self.readyFrame.setLayout(self.readyLayout)

        # Initial state

        self.readyInitialGroup = QtWidgets.QGroupBox(getMessage("readiness-title"))
        self.readyInitialLayout = QtWidgets.QVBoxLayout()
        self.readyInitialGroup.setLayout(self.readyInitialLayout)
        self.readyatstartCheckbox = QCheckBox(getMessage("readyatstart-label"))
        self.readyatstartCheckbox.setObjectName("readyAtStart")
        self.readyInitialLayout.addWidget(self.readyatstartCheckbox)
        self.readyLayout.addWidget(self.readyInitialGroup)

        # Automatically pausing
        self.readyPauseGroup = QtWidgets.QGroupBox(getMessage("pausing-title"))
        self.readyPauseLayout = QtWidgets.QVBoxLayout()
        self.readyPauseGroup.setLayout(self.readyPauseLayout)
        self.pauseonleaveCheckbox = QCheckBox(getMessage("pauseonleave-label"))
        self.pauseonleaveCheckbox.setObjectName("pauseOnLeave")
        self.readyPauseLayout.addWidget(self.pauseonleaveCheckbox)
        self.readyLayout.addWidget(self.readyPauseGroup)

        # Unpausing
        self.readyUnpauseGroup = QtWidgets.QGroupBox(getMessage("unpause-title"))
        self.readyUnpauseLayout = QtWidgets.QVBoxLayout()
        self.readyUnpauseGroup.setLayout(self.readyUnpauseLayout)
        self.readyUnpauseButtonGroup = QButtonGroup()
        self.unpauseIfAlreadyReadyOption = QRadioButton(getMessage("unpause-ifalreadyready-option"))
        self.readyUnpauseButtonGroup.addButton(self.unpauseIfAlreadyReadyOption)
        self.unpauseIfAlreadyReadyOption.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.unpauseIfAlreadyReadyOption.setObjectName("unpause-ifalreadyready" + constants.CONFIG_NAME_MARKER + "unpauseAction" + constants.CONFIG_VALUE_MARKER + constants.UNPAUSE_IFALREADYREADY_MODE)
        self.readyUnpauseLayout.addWidget(self.unpauseIfAlreadyReadyOption)
        self.unpauseIfOthersReadyOption = QRadioButton(getMessage("unpause-ifothersready-option"))
        self.readyUnpauseButtonGroup.addButton(self.unpauseIfOthersReadyOption)
        self.unpauseIfOthersReadyOption.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.unpauseIfOthersReadyOption.setObjectName("unpause-ifothersready" + constants.CONFIG_NAME_MARKER + "unpauseAction" + constants.CONFIG_VALUE_MARKER + constants.UNPAUSE_IFOTHERSREADY_MODE)
        self.readyUnpauseLayout.addWidget(self.unpauseIfOthersReadyOption)
        self.unpauseIfMinUsersReadyOption = QRadioButton(getMessage("unpause-ifminusersready-option"))
        self.readyUnpauseButtonGroup.addButton(self.unpauseIfMinUsersReadyOption)
        self.unpauseIfMinUsersReadyOption.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.unpauseIfMinUsersReadyOption.setObjectName("unpause-ifminusersready" + constants.CONFIG_NAME_MARKER + "unpauseAction" + constants.CONFIG_VALUE_MARKER + constants.UNPAUSE_IFMINUSERSREADY_MODE)
        self.readyUnpauseLayout.addWidget(self.unpauseIfMinUsersReadyOption)
        self.unpauseAlwaysUnpauseOption = QRadioButton(getMessage("unpause-always"))
        self.readyUnpauseButtonGroup.addButton(self.unpauseAlwaysUnpauseOption)
        self.unpauseAlwaysUnpauseOption.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.unpauseAlwaysUnpauseOption.setObjectName("unpause-always" + constants.CONFIG_NAME_MARKER + "unpauseAction" + constants.CONFIG_VALUE_MARKER + constants.UNPAUSE_ALWAYS_MODE)
        self.readyUnpauseLayout.addWidget(self.unpauseAlwaysUnpauseOption)
        self.readyLayout.addWidget(self.readyUnpauseGroup)

        self.readyLayout.setAlignment(Qt.AlignTop)
        self.stackedLayout.addWidget(self.readyFrame)

    def addMiscTab(self):
        self.miscFrame = QtWidgets.QFrame()
        self.miscLayout = QtWidgets.QVBoxLayout()
        self.miscFrame.setLayout(self.miscLayout)

        self.coreSettingsGroup = QtWidgets.QGroupBox(getMessage("core-behaviour-title"))
        self.coreSettingsLayout = QtWidgets.QGridLayout()
        self.coreSettingsGroup.setLayout(self.coreSettingsLayout)

        ### Privacy:

        self.filenameprivacyLabel = QLabel(getMessage("filename-privacy-label"), self)
        self.filenameprivacyButtonGroup = QButtonGroup()
        self.filenameprivacySendRawOption = QRadioButton(getMessage("privacy-sendraw-option"))
        self.filenameprivacySendHashedOption = QRadioButton(getMessage("privacy-sendhashed-option"))
        self.filenameprivacyDontSendOption = QRadioButton(getMessage("privacy-dontsend-option"))
        self.filenameprivacyButtonGroup.addButton(self.filenameprivacySendRawOption)
        self.filenameprivacyButtonGroup.addButton(self.filenameprivacySendHashedOption)
        self.filenameprivacyButtonGroup.addButton(self.filenameprivacyDontSendOption)

        self.filesizeprivacyLabel = QLabel(getMessage("filesize-privacy-label"), self)
        self.filesizeprivacyButtonGroup = QButtonGroup()
        self.filesizeprivacySendRawOption = QRadioButton(getMessage("privacy-sendraw-option"))
        self.filesizeprivacySendHashedOption = QRadioButton(getMessage("privacy-sendhashed-option"))
        self.filesizeprivacyDontSendOption = QRadioButton(getMessage("privacy-dontsend-option"))
        self.filesizeprivacyButtonGroup.addButton(self.filesizeprivacySendRawOption)
        self.filesizeprivacyButtonGroup.addButton(self.filesizeprivacySendHashedOption)
        self.filesizeprivacyButtonGroup.addButton(self.filesizeprivacyDontSendOption)

        self.filenameprivacyLabel.setObjectName("filename-privacy")
        self.filenameprivacySendRawOption.setObjectName("privacy-sendraw" + constants.CONFIG_NAME_MARKER + "filenamePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_SENDRAW_MODE)
        self.filenameprivacySendHashedOption.setObjectName("privacy-sendhashed" + constants.CONFIG_NAME_MARKER + "filenamePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_SENDHASHED_MODE)
        self.filenameprivacyDontSendOption.setObjectName("privacy-dontsend" + constants.CONFIG_NAME_MARKER + "filenamePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_DONTSEND_MODE)
        self.filesizeprivacyLabel.setObjectName("filesize-privacy")
        self.filesizeprivacySendRawOption.setObjectName("privacy-sendraw" + constants.CONFIG_NAME_MARKER + "filesizePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_SENDRAW_MODE)
        self.filesizeprivacySendHashedOption.setObjectName("privacy-sendhashed" + constants.CONFIG_NAME_MARKER + "filesizePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_SENDHASHED_MODE)
        self.filesizeprivacyDontSendOption.setObjectName("privacy-dontsend" + constants.CONFIG_NAME_MARKER + "filesizePrivacyMode" + constants.CONFIG_VALUE_MARKER + constants.PRIVACY_DONTSEND_MODE)

        self.coreSettingsLayout.addWidget(self.filenameprivacyLabel, 3, 0)
        self.coreSettingsLayout.addWidget(self.filenameprivacySendRawOption, 3, 1, Qt.AlignLeft)
        self.coreSettingsLayout.addWidget(self.filenameprivacySendHashedOption, 3, 2, Qt.AlignLeft)
        self.coreSettingsLayout.addWidget(self.filenameprivacyDontSendOption, 3, 3, Qt.AlignLeft)
        self.coreSettingsLayout.addWidget(self.filesizeprivacyLabel, 4, 0)
        self.coreSettingsLayout.addWidget(self.filesizeprivacySendRawOption, 4, 1, Qt.AlignLeft)
        self.coreSettingsLayout.addWidget(self.filesizeprivacySendHashedOption, 4, 2, Qt.AlignLeft)
        self.coreSettingsLayout.addWidget(self.filesizeprivacyDontSendOption, 4, 3, Qt.AlignLeft)

        ## Syncplay internals

        self.internalSettingsGroup = QtWidgets.QGroupBox(getMessage("syncplay-internals-title"))
        self.internalSettingsLayout = QtWidgets.QVBoxLayout()
        self.internalSettingsGroup.setLayout(self.internalSettingsLayout)

        self.alwaysshowCheckbox = QCheckBox(getMessage("forceguiprompt-label"))
        self.alwaysshowCheckbox.setObjectName(constants.INVERTED_STATE_MARKER + "forceGuiPrompt")
        self.internalSettingsLayout.addWidget(self.alwaysshowCheckbox)

        self.automaticupdatesCheckbox = QCheckBox(getMessage("checkforupdatesautomatically-label"))
        self.automaticupdatesCheckbox.setObjectName("checkForUpdatesAutomatically")
        self.internalSettingsLayout.addWidget(self.automaticupdatesCheckbox)

        self.autosaveJoinsToListCheckbox = QCheckBox(getMessage("autosavejoinstolist-label"))
        self.autosaveJoinsToListCheckbox.setObjectName("autosaveJoinsToList")
        self.internalSettingsLayout.addWidget(self.autosaveJoinsToListCheckbox)

        ## Media path directories

        self.mediasearchSettingsGroup = QtWidgets.QGroupBox(getMessage("syncplay-mediasearchdirectories-title"))
        self.mediasearchSettingsLayout = QtWidgets.QVBoxLayout()
        self.mediasearchSettingsGroup.setLayout(self.mediasearchSettingsLayout)

        self.mediasearchTextEdit = QPlainTextEdit(utils.getListAsMultilineString(self.mediaSearchDirectories))
        self.mediasearchTextEdit.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "mediasearcdirectories-arguments")
        self.mediasearchTextEdit.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.mediasearchSettingsLayout.addWidget(self.mediasearchTextEdit)
        self.mediasearchSettingsGroup.setMaximumHeight(self.mediasearchSettingsGroup.minimumSizeHint().height())

        self.miscLayout.addWidget(self.coreSettingsGroup)
        self.miscLayout.addWidget(self.internalSettingsGroup)
        self.miscLayout.addWidget(self.mediasearchSettingsGroup)
        self.miscLayout.setAlignment(Qt.AlignTop)
        self.stackedLayout.addWidget(self.miscFrame)

    def addSyncTab(self):
        self.syncSettingsFrame = QtWidgets.QFrame()
        self.syncSettingsLayout = QtWidgets.QVBoxLayout()

        self.desyncSettingsGroup = QtWidgets.QGroupBox(getMessage("sync-otherslagging-title"))
        self.desyncOptionsFrame = QtWidgets.QFrame()
        self.desyncSettingsOptionsLayout = QtWidgets.QHBoxLayout()
        config = self.config

        self.slowdownCheckbox = QCheckBox(getMessage("slowondesync-label"))
        self.slowdownCheckbox.setObjectName("slowOnDesync")
        self.rewindCheckbox = QCheckBox(getMessage("rewindondesync-label"))
        self.rewindCheckbox.setObjectName("rewindOnDesync")
        self.fastforwardCheckbox = QCheckBox(getMessage("fastforwardondesync-label"))
        self.fastforwardCheckbox.setObjectName("fastforwardOnDesync")

        self.desyncSettingsLayout = QtWidgets.QGridLayout()
        self.desyncSettingsLayout.setSpacing(2)
        self.desyncFrame = QtWidgets.QFrame()
        self.desyncFrame.setLineWidth(0)
        self.desyncFrame.setMidLineWidth(0)

        self.desyncSettingsLayout.addWidget(self.slowdownCheckbox, 0, 0, 1, 2, Qt.AlignLeft)
        self.desyncSettingsLayout.addWidget(self.rewindCheckbox, 1, 0, 1, 2, Qt.AlignLeft)

        self.desyncSettingsLayout.setAlignment(Qt.AlignLeft)
        self.desyncSettingsGroup.setLayout(self.desyncSettingsLayout)
        self.desyncSettingsOptionsLayout.addWidget(self.desyncFrame)

        self.desyncFrame.setLayout(self.syncSettingsLayout)

        self.othersyncSettingsGroup = QtWidgets.QGroupBox(getMessage("sync-youlaggging-title"))
        self.othersyncOptionsFrame = QtWidgets.QFrame()
        self.othersyncSettingsLayout = QtWidgets.QGridLayout()

        self.dontslowwithmeCheckbox = QCheckBox(getMessage("dontslowdownwithme-label"))
        self.dontslowwithmeCheckbox.setObjectName("dontSlowDownWithMe")

        self.othersyncSettingsLayout.addWidget(self.dontslowwithmeCheckbox, 2, 0, 1, 2, Qt.AlignLeft)

        self.othersyncSettingsLayout.setAlignment(Qt.AlignLeft)
        self.othersyncSettingsLayout.addWidget(self.fastforwardCheckbox, 3, 0, 1, 2, Qt.AlignLeft)

        ## Trusted domains

        self.trusteddomainsSettingsGroup = QtWidgets.QGroupBox(getMessage("syncplay-trusteddomains-title"))
        self.trusteddomainsSettingsLayout = QtWidgets.QVBoxLayout()
        self.trusteddomainsSettingsGroup.setLayout(self.trusteddomainsSettingsLayout)

        self.trusteddomainsTextEdit = QPlainTextEdit(utils.getListAsMultilineString(self.trustedDomains))
        self.trusteddomainsTextEdit.setObjectName(constants.LOAD_SAVE_MANUALLY_MARKER + "trusteddomains-arguments")
        self.trusteddomainsTextEdit.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.trusteddomainsSettingsLayout.addWidget(self.trusteddomainsTextEdit)
        self.trusteddomainsSettingsGroup.setMaximumHeight(self.trusteddomainsSettingsGroup.minimumSizeHint().height())

        self.othersyncSettingsGroup.setLayout(self.othersyncSettingsLayout)
        self.othersyncSettingsGroup.setMaximumHeight(self.othersyncSettingsGroup.minimumSizeHint().height())
        self.syncSettingsLayout.addWidget(self.othersyncSettingsGroup)
        self.syncSettingsLayout.addWidget(self.desyncSettingsGroup)
        self.syncSettingsLayout.addWidget(self.trusteddomainsSettingsGroup)
        self.syncSettingsFrame.setLayout(self.syncSettingsLayout)
        self.desyncSettingsGroup.setMaximumHeight(self.desyncSettingsGroup.minimumSizeHint().height())
        self.syncSettingsLayout.setAlignment(Qt.AlignTop)
        self.stackedLayout.addWidget(self.syncSettingsFrame)

    def addChatTab(self):
        self.chatFrame = QtWidgets.QFrame()
        self.chatLayout = QtWidgets.QVBoxLayout()
        self.chatLayout.setAlignment(Qt.AlignTop)

        # Input
        self.chatInputGroup = QtWidgets.QGroupBox(getMessage("chat-title"))
        self.chatInputLayout = QtWidgets.QGridLayout()
        self.chatLayout.addWidget(self.chatInputGroup)
        self.chatInputGroup.setLayout(self.chatInputLayout)
        self.chatInputEnabledCheckbox = QCheckBox(getMessage("chatinputenabled-label"))
        self.chatInputEnabledCheckbox.setObjectName("chatInputEnabled")
        self.chatInputLayout.addWidget(self.chatInputEnabledCheckbox, 1, 0, 1, 1, Qt.AlignLeft)

        self.chatDirectInputCheckbox = QCheckBox(getMessage("chatdirectinput-label"))
        self.chatDirectInputCheckbox.setObjectName("chatDirectInput")
        self.chatDirectInputCheckbox.setStyleSheet(
            constants.STYLE_SUBCHECKBOX.format(self.posixresourcespath + "chevrons_right.png"))
        self.chatInputLayout.addWidget(self.chatDirectInputCheckbox, 2, 0, 1, 1, Qt.AlignLeft)

        self.inputFontLayout = QtWidgets.QHBoxLayout()
        self.inputFontLayout.setContentsMargins(0, 0, 0, 0)
        self.inputFontFrame = QtWidgets.QFrame()
        self.inputFontFrame.setLayout(self.inputFontLayout)
        self.inputFontFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.chatFontLabel = QLabel(getMessage("chatinputfont-label"), self)
        self.chatFontLabel.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(self.posixresourcespath + "chevrons_right.png"))
        self.chatFontLabel.setObjectName("font-label")
        self.chatInputFontButton = QtWidgets.QPushButton(getMessage("chatfont-label"))
        self.chatInputFontButton.setObjectName("set-input-font")
        self.chatInputFontButtonGroup = QtWidgets.QButtonGroup()
        self.chatInputFontButtonGroup.addButton(self.chatInputFontButton)
        self.chatInputFontButton.released.connect(lambda: self.fontDialog("chatInput"))
        self.chatInputColourButton = QtWidgets.QPushButton(getMessage("chatcolour-label"))
        self.chatInputColourButton.setObjectName("set-input-colour")
        self.chatInputColourButtonGroup = QtWidgets.QButtonGroup()
        self.chatInputColourButtonGroup.addButton(self.chatInputColourButton)
        self.chatInputColourButton.released.connect(lambda: self.colourDialog("chatInput"))
        self.inputFontLayout.addWidget(self.chatFontLabel, Qt.AlignLeft)
        self.inputFontLayout.addWidget(self.chatInputFontButton, Qt.AlignLeft)
        self.inputFontLayout.addWidget(self.chatInputColourButton, Qt.AlignLeft)
        self.chatInputLayout.addWidget(self.inputFontFrame, 3, 0, 1, 3, Qt.AlignLeft)

        self.chatInputPositionFrame = QtWidgets.QFrame()
        self.chatInputPositionLayout = QtWidgets.QHBoxLayout()
        self.chatInputPositionLayout.setContentsMargins(0, 0, 0, 0)
        self.chatInputPositionFrame.setLayout(self.chatInputPositionLayout)
        self.chatInputPositionFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.chatInputPositionLabel = QLabel(getMessage("chatinputposition-label"), self)
        self.chatInputPositionLabel.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(self.posixresourcespath + "chevrons_right.png"))
        self.chatInputPositionGroup = QButtonGroup()
        self.chatInputTopOption = QRadioButton(getMessage("chat-top-option"))
        self.chatInputMiddleOption = QRadioButton(getMessage("chat-middle-option"))
        self.chatInputBottomOption = QRadioButton(getMessage("chat-bottom-option"))
        self.chatInputPositionGroup.addButton(self.chatInputTopOption)
        self.chatInputPositionGroup.addButton(self.chatInputMiddleOption)
        self.chatInputPositionGroup.addButton(self.chatInputBottomOption)

        self.chatInputPositionLabel.setObjectName("chatinputposition")
        self.chatInputTopOption.setObjectName("chatinputposition-top" + constants.CONFIG_NAME_MARKER + "chatInputPosition" + constants.CONFIG_VALUE_MARKER + constants.INPUT_POSITION_TOP)
        self.chatInputMiddleOption.setObjectName("chatinputposition-middle" + constants.CONFIG_NAME_MARKER + "chatInputPosition" + constants.CONFIG_VALUE_MARKER + constants.INPUT_POSITION_MIDDLE)
        self.chatInputBottomOption.setObjectName("chatinputposition-bottom" + constants.CONFIG_NAME_MARKER + "chatInputPosition" + constants.CONFIG_VALUE_MARKER + constants.INPUT_POSITION_BOTTOM)

        self.chatInputPositionLayout.addWidget(self.chatInputPositionLabel)
        self.chatInputPositionLayout.addWidget(self.chatInputTopOption)
        self.chatInputPositionLayout.addWidget(self.chatInputMiddleOption)
        self.chatInputPositionLayout.addWidget(self.chatInputBottomOption)
        self.chatInputLayout.addWidget(self.chatInputPositionFrame)

        self.subitems['chatInputEnabled'] = [self.chatInputPositionLabel.objectName(), self.chatInputTopOption.objectName(),
                                             self.chatInputMiddleOption.objectName(), self.chatInputBottomOption.objectName(),
                                             self.chatInputFontButton.objectName(), self.chatFontLabel.objectName(),
                                             self.chatInputColourButton.objectName(), self.chatDirectInputCheckbox.objectName()]
        # Output
        self.chatOutputGroup = QtWidgets.QGroupBox(getMessage("chatoutputheader-label"))
        self.chatOutputLayout = QtWidgets.QGridLayout()
        self.chatLayout.addWidget(self.chatOutputGroup)
        self.chatOutputGroup.setLayout(self.chatOutputLayout)
        self.chatOutputEnabledCheckbox = QCheckBox(getMessage("chatoutputenabled-label"))
        self.chatOutputEnabledCheckbox.setObjectName("chatOutputEnabled")
        self.chatOutputLayout.addWidget(self.chatOutputEnabledCheckbox, 1, 0, 1, 1, Qt.AlignLeft)

        self.outputFontLayout = QtWidgets.QHBoxLayout()
        self.outputFontLayout.setContentsMargins(0, 0, 0, 0)
        self.outputFontFrame = QtWidgets.QFrame()
        self.outputFontFrame.setLayout(self.outputFontLayout)
        self.outputFontFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.chatOutputFontLabel = QLabel(getMessage("chatoutputfont-label"), self)
        self.chatOutputFontLabel.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(self.posixresourcespath + "chevrons_right.png"))
        self.chatOutputFontLabel.setObjectName("font-output-label")
        self.chatOutputFontButton = QtWidgets.QPushButton(getMessage("chatfont-label"))
        self.chatOutputFontButton.setObjectName("set-output-font")
        self.chatOutputFontButtonGroup = QtWidgets.QButtonGroup()
        self.chatOutputFontButtonGroup.addButton(self.chatOutputFontButton)
        self.chatOutputFontButton.released.connect(lambda: self.fontDialog("chatOutput"))
        self.chatOutputColourButton = QtWidgets.QPushButton(getMessage("chatcolour-label"))
        self.outputFontLayout.addWidget(self.chatOutputFontLabel, Qt.AlignLeft)
        self.outputFontLayout.addWidget(self.chatOutputFontButton, Qt.AlignLeft)
        self.chatOutputLayout.addWidget(self.outputFontFrame, 2, 0, 1, 3, Qt.AlignLeft)

        self.chatOutputModeLabel = QLabel(getMessage("chatoutputposition-label"), self)
        self.chatOutputModeLabel.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(self.posixresourcespath + "chevrons_right.png"))
        self.chatOutputModeGroup = QButtonGroup()
        self.chatOutputChatroomOption = QRadioButton(getMessage("chat-chatroom-option"))
        self.chatOutputScrollingOption = QRadioButton(getMessage("chat-scrolling-option"))
        self.chatOutputModeGroup.addButton(self.chatOutputChatroomOption)
        self.chatOutputModeGroup.addButton(self.chatOutputScrollingOption)

        self.chatOutputModeLabel.setObjectName("chatoutputmode")
        self.chatOutputChatroomOption.setObjectName("chatoutputmode-chatroom" + constants.CONFIG_NAME_MARKER + "chatOutputMode" + constants.CONFIG_VALUE_MARKER + constants.CHATROOM_MODE)
        self.chatOutputScrollingOption.setObjectName("chatoutputmode-scrolling" + constants.CONFIG_NAME_MARKER + "chatOutputMode" + constants.CONFIG_VALUE_MARKER + constants.SCROLLING_MODE)

        self.chatOutputModeFrame = QtWidgets.QFrame()
        self.chatOutputModeLayout = QtWidgets.QHBoxLayout()
        self.chatOutputModeLayout.setContentsMargins(0, 0, 0, 0)
        self.chatOutputModeFrame.setLayout(self.chatOutputModeLayout)
        self.chatOutputModeFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.chatOutputModeLayout.addWidget(self.chatOutputModeLabel)
        self.chatOutputModeLayout.addWidget(self.chatOutputChatroomOption)
        self.chatOutputModeLayout.addWidget(self.chatOutputScrollingOption)
        self.chatOutputLayout.addWidget(self.chatOutputModeFrame)

        self.subitems['chatOutputEnabled'] = [self.chatOutputModeLabel.objectName(), self.chatOutputChatroomOption.objectName(),
                                              self.chatOutputScrollingOption.objectName(), self.chatOutputFontButton.objectName(),
                                              self.chatOutputFontLabel.objectName()]
        # chatFrame
        self.chatFrame.setLayout(self.chatLayout)
        self.stackedLayout.addWidget(self.chatFrame)

    def fontDialog(self, configName):
            font = QtGui.QFont()
            font.setFamily(self.config[configName + "FontFamily"])
            font.setPointSize(self.config[configName + "RelativeFontSize"])
            font.setWeight(self.config[configName + "FontWeight"])
            font.setUnderline(self.config[configName + "FontUnderline"])
            ok, value = QtWidgets.QFontDialog.getFont(font)
            if ok:
                self.config[configName + "FontFamily"] = value.family()
                self.config[configName + "RelativeFontSize"] = value.pointSize()
                self.config[configName + "FontWeight"] = value.weight()
                self.config[configName + "FontUnderline"] = value.underline()

    def colourDialog(self, configName):
            oldColour = QtGui.QColor()
            oldColour.setNamedColor(self.config[configName + "FontColor"])
            colour = QtWidgets.QColorDialog.getColor(oldColour, self)
            if colour.isValid():
                self.config[configName + "FontColor"] = colour.name()

    def addMessageTab(self):
        self.messageFrame = QtWidgets.QFrame()
        self.messageLayout = QtWidgets.QVBoxLayout()
        self.messageLayout.setAlignment(Qt.AlignTop)

        # OSD
        self.osdSettingsGroup = QtWidgets.QGroupBox(getMessage("messages-osd-title"))
        self.osdSettingsLayout = QtWidgets.QVBoxLayout()
        self.osdSettingsFrame = QtWidgets.QFrame()

        self.showOSDCheckbox = QCheckBox(getMessage("showosd-label"))
        self.showOSDCheckbox.setObjectName("showOSD")
        self.osdSettingsLayout.addWidget(self.showOSDCheckbox)

        self.showSameRoomOSDCheckbox = QCheckBox(getMessage("showsameroomosd-label"))
        self.showSameRoomOSDCheckbox.setObjectName("showSameRoomOSD")
        self.showSameRoomOSDCheckbox.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.osdSettingsLayout.addWidget(self.showSameRoomOSDCheckbox)

        self.showNonControllerOSDCheckbox = QCheckBox(getMessage("shownoncontrollerosd-label"))
        self.showNonControllerOSDCheckbox.setObjectName("showNonControllerOSD")
        self.showNonControllerOSDCheckbox.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.osdSettingsLayout.addWidget(self.showNonControllerOSDCheckbox)

        self.showDifferentRoomOSDCheckbox = QCheckBox(getMessage("showdifferentroomosd-label"))
        self.showDifferentRoomOSDCheckbox.setObjectName("showDifferentRoomOSD")
        self.showDifferentRoomOSDCheckbox.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.osdSettingsLayout.addWidget(self.showDifferentRoomOSDCheckbox)

        self.slowdownOSDCheckbox = QCheckBox(getMessage("showslowdownosd-label"))
        self.slowdownOSDCheckbox.setObjectName("showSlowdownOSD")
        self.slowdownOSDCheckbox.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.osdSettingsLayout.addWidget(self.slowdownOSDCheckbox)

        self.showOSDWarningsCheckbox = QCheckBox(getMessage("showosdwarnings-label"))
        self.showOSDWarningsCheckbox.setObjectName("showOSDWarnings")
        self.showOSDWarningsCheckbox.setStyleSheet(constants.STYLE_SUBCHECKBOX.format(posixresourcespath + "chevrons_right.png"))
        self.osdSettingsLayout.addWidget(self.showOSDWarningsCheckbox)

        self.subitems['showOSD'] = ["showSameRoomOSD", "showDifferentRoomOSD", "showSlowdownOSD", "showOSDWarnings", "showNonControllerOSD"]

        self.osdSettingsGroup.setLayout(self.osdSettingsLayout)
        self.osdSettingsGroup.setMaximumHeight(self.osdSettingsGroup.minimumSizeHint().height())
        self.osdSettingsLayout.setAlignment(Qt.AlignTop)
        self.messageLayout.addWidget(self.osdSettingsGroup)

        # Other display

        self.displaySettingsGroup = QtWidgets.QGroupBox(getMessage("messages-other-title"))
        self.displaySettingsLayout = QtWidgets.QVBoxLayout()
        self.displaySettingsFrame = QtWidgets.QFrame()

        self.showDurationNotificationCheckbox = QCheckBox(getMessage("showdurationnotification-label"))
        self.showDurationNotificationCheckbox.setObjectName("showDurationNotification")
        self.displaySettingsLayout.addWidget(self.showDurationNotificationCheckbox)

        self.languageFrame = QtWidgets.QFrame()
        self.languageLayout = QtWidgets.QHBoxLayout()
        self.languageLayout.setContentsMargins(0, 0, 0, 0)
        self.languageFrame.setLayout(self.languageLayout)
        self.languageFrame.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.languageLabel = QLabel(getMessage("language-label"), self)
        self.languageCombobox = QtWidgets.QComboBox(self)
        self.languageCombobox.addItem(getMessage("automatic-language").format(getMessage("LANGUAGE", getInitialLanguage())))

        self.languages = getLanguages()
        for lang in self.languages:
            self.languageCombobox.addItem(self.languages[lang], lang)
            if lang == self.config['language']:
                self.languageCombobox.setCurrentIndex(self.languageCombobox.count()-1)
        self.languageCombobox.currentIndexChanged.connect(self.languageChanged)
        self.languageLayout.addWidget(self.languageLabel, 1, Qt.AlignLeft)
        self.languageLayout.addWidget(self.languageCombobox, 1, Qt.AlignLeft)
        self.displaySettingsLayout.addWidget(self.languageFrame)

        self.languageLabel.setObjectName("language")
        self.languageCombobox.setObjectName("language")
        self.languageFrame.setMaximumWidth(self.languageFrame.minimumSizeHint().width())

        self.displaySettingsGroup.setLayout(self.displaySettingsLayout)
        self.displaySettingsGroup.setMaximumHeight(self.displaySettingsGroup.minimumSizeHint().height())
        self.messageLayout.addWidget(self.displaySettingsGroup)

        # messageFrame
        self.messageFrame.setLayout(self.messageLayout)
        self.stackedLayout.addWidget(self.messageFrame)

    def addBottomLayout(self):
        config = self.config

        self.bottomButtonFrame = QtWidgets.QFrame()
        self.bottomButtonLayout = QtWidgets.QHBoxLayout()
        self.helpButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'help.png'), getMessage("help-label"))
        self.helpButton.setObjectName("help")
        self.helpButton.setMaximumSize(self.helpButton.sizeHint())
        self.helpButton.released.connect(self.openHelp)

        self.resetButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'cog_delete.png'), getMessage("reset-label"))
        self.resetButton.setMaximumSize(self.resetButton.sizeHint())
        self.resetButton.setObjectName("reset")
        self.resetButton.released.connect(self.resetSettings)

        self.runButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'accept.png'), getMessage("run-label"))
        self.runButton.released.connect(self._runWithoutStoringConfig)
        self.runButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'accept.png'), getMessage("run-label"))
        self.runButton.pressed.connect(self._runWithoutStoringConfig)
        self.runButton.setToolTip(getMessage("nostore-tooltip"))
        self.storeAndRunButton = QtWidgets.QPushButton(QtGui.QIcon(resourcespath + 'accept.png'), getMessage("storeandrun-label"))
        self.storeAndRunButton.released.connect(self._saveDataAndLeave)
        self.bottomButtonLayout.addWidget(self.helpButton)
        self.bottomButtonLayout.addWidget(self.resetButton)
        self.bottomButtonLayout.addWidget(self.runButton)
        self.bottomButtonLayout.addWidget(self.storeAndRunButton)
        self.bottomButtonFrame.setLayout(self.bottomButtonLayout)
        if isMacOS():
            self.bottomButtonLayout.setContentsMargins(15, 0, 15, 0)
        else:
            self.bottomButtonLayout.setContentsMargins(5, 0, 5, 0)
        self.mainLayout.addWidget(self.bottomButtonFrame, 1, 0, 1, 2)

        self.bottomCheckboxFrame = QtWidgets.QFrame()
        if isMacOS():
            self.bottomCheckboxFrame.setContentsMargins(3, 0, 6, 0)
        else:
            self.bottomCheckboxFrame.setContentsMargins(0, 0, 0, 0)
        self.bottomCheckboxLayout = QtWidgets.QGridLayout()
        self.alwaysshowCheckbox = QCheckBox(getMessage("forceguiprompt-label"))

        self.enableplaylistsCheckbox = QCheckBox(getMessage("sharedplaylistenabled-label"))
        self.bottomCheckboxLayout.addWidget(self.showmoreCheckbox)
        self.bottomCheckboxLayout.addWidget(self.enableplaylistsCheckbox, 0, 2, Qt.AlignRight)
        self.alwaysshowCheckbox.setObjectName(constants.INVERTED_STATE_MARKER + "forceGuiPrompt")
        self.enableplaylistsCheckbox.setObjectName("sharedPlaylistEnabled")
        self.bottomCheckboxFrame.setLayout(self.bottomCheckboxLayout)

        self.mainLayout.addWidget(self.bottomCheckboxFrame, 2, 0, 1, 2)

    def tabList(self):
        self.tabListLayout = QtWidgets.QHBoxLayout()
        self.tabListFrame = QtWidgets.QFrame()
        self.tabListWidget = QtWidgets.QListWidget()
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "house.png"), getMessage("basics-label")))
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "control_pause_blue.png"), getMessage("readiness-label")))
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "film_link.png"), getMessage("sync-label")))
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "user_comment.png"), getMessage("chat-label")))
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "error.png"), getMessage("messages-label")))
        self.tabListWidget.addItem(QtWidgets.QListWidgetItem(QtGui.QIcon(resourcespath + "cog.png"), getMessage("misc-label")))
        self.tabListLayout.addWidget(self.tabListWidget)
        self.tabListFrame.setLayout(self.tabListLayout)
        self.tabListFrame.setFixedWidth(self.tabListFrame.minimumSizeHint().width() + constants.TAB_PADDING)
        self.tabListWidget.setStyleSheet(constants.STYLE_TABLIST)

        self.tabListWidget.currentItemChanged.connect(self.tabChange)
        self.tabListWidget.itemClicked.connect(self.tabChange)
        self.tabListWidget.itemPressed.connect(self.tabChange)
        self.mainLayout.addWidget(self.tabListFrame, 0, 0, 1, 1)

    def ensureTabListIsVisible(self):
        self.stackedFrame.setFixedWidth(self.stackedFrame.width())
        while self.tabListWidget.horizontalScrollBar().isVisible() and self.tabListFrame.width() < constants.MAXIMUM_TAB_WIDTH:
            self.tabListFrame.setFixedWidth(self.tabListFrame.width()+1)

    def tabChange(self):
        self.setFocus()
        self.stackedLayout.setCurrentIndex(self.tabListWidget.currentRow())

    def resetSettings(self):
        self.clearGUIData(leaveMore=True)
        self.config['resetConfig'] = True
        self.pressedclosebutton = False
        self.close()
        self.closed.emit()

    def showEvent(self, *args, **kwargs):
        self.ensureTabListIsVisible()
        self.setFixedWidth(self.minimumSizeHint().width())

    def clearGUIData(self, leaveMore=False):
        settings = QSettings("Syncplay", "PlayerList")
        settings.clear()
        settings = QSettings("Syncplay", "MediaBrowseDialog")
        settings.clear()
        settings = QSettings("Syncplay", "MainWindow")
        settings.clear()
        settings = QSettings("Syncplay", "Interface")
        settings.beginGroup("Update")
        settings.setValue("lastChecked", None)
        settings.setValue("lastCheckedQt", None)
        settings.endGroup()
        settings.beginGroup("PublicServerList")
        settings.setValue("publicServers", None)
        settings.endGroup()
        if not leaveMore:
            settings = QSettings("Syncplay", "MoreSettings")
            settings.clear()
        self.datacleared = True

    def populateEmptyServerList(self):
        if self.publicServers is None:
            if self.config["checkForUpdatesAutomatically"] == True:
                self.updateServerList()
            else:
                currentServer = self.hostCombobox.currentText()
                self.publicServers = constants.FALLBACK_PUBLIC_SYNCPLAY_SERVERS
                i = 0
                for server in self.publicServers:
                    self.hostCombobox.addItem(server[1])
                    self.hostCombobox.setItemData(i, server[0], Qt.ToolTipRole)
                    if not server[1] in self.publicServerAddresses:
                        self.publicServerAddresses.append(server[1])
                    i += 1
                self.hostCombobox.setEditText(currentServer)

    def updatePasswordVisibilty(self):
        if (self.hostCombobox.currentText() == "" and self.serverpassTextbox.text() == "") or str(self.hostCombobox.currentText()) in self.publicServerAddresses:
            self.serverpassTextbox.setDisabled(True)
            self.serverpassTextbox.setReadOnly(True)
            if self.serverpassTextbox.text() != "":
                self.storedPassword = self.serverpassTextbox.text()
            self.serverpassTextbox.setText("")
        else:
            self.serverpassTextbox.setEnabled(True)
            self.serverpassTextbox.setReadOnly(False)
            self.serverpassTextbox.setText(self.storedPassword)

    def createMenubar(self):
        self.menuBar = QtWidgets.QMenuBar()

        # Edit menu
        self.editMenu = QtWidgets.QMenu(getMessage("edit-menu-label"), self)

        self.cutAction = self.editMenu.addAction(getMessage("cut-menu-label"))
        self.cutAction.setShortcuts(QtGui.QKeySequence.Cut)

        self.copyAction = self.editMenu.addAction(getMessage("copy-menu-label"))
        self.copyAction.setShortcuts(QtGui.QKeySequence.Copy)

        self.pasteAction = self.editMenu.addAction(getMessage("paste-menu-label"))
        self.pasteAction.setShortcuts(QtGui.QKeySequence.Paste)

        self.selectAction = self.editMenu.addAction(getMessage("selectall-menu-label"))
        self.selectAction.setShortcuts(QtGui.QKeySequence.SelectAll)

        self.editMenu.addSeparator()

        self.menuBar.addMenu(self.editMenu)

        self.mainLayout.setMenuBar(self.menuBar)

    def __init__(self, config, playerpaths, error, defaultConfig):
        self.config = config
        self.defaultConfig = defaultConfig
        self.playerpaths = playerpaths
        self.datacleared = False
        self.config['resetConfig'] = False
        self.subitems = {}
        self.publicServers = None
        self.publicServerAddresses = []

        self._playerProbeThread = GetPlayerIconThread()
        self._playerProbeThread.done.connect(self._updateExecutableIcon)
        self._playerProbeThread.start()

        if self.config['clearGUIData'] == True:
            self.config['clearGUIData'] = False
            self.clearGUIData()

        self.QtWidgets = QtWidgets
        self.QtGui = QtGui
        self.error = error
        if isWindows():
            resourcespath = utils.findWorkingDir() + "\\resources\\"
        else:
            resourcespath = utils.findWorkingDir() + "/resources/"
        self.posixresourcespath = utils.findWorkingDir().replace("\\", "/") + "/resources/"
        self.resourcespath = resourcespath

        super(ConfigDialog, self).__init__()

        self.setWindowTitle(getMessage("config-window-title"))
        self.setWindowFlags(self.windowFlags() & Qt.WindowCloseButtonHint & ~Qt.WindowContextHelpButtonHint)
        self.setWindowIcon(QtGui.QIcon(resourcespath + "syncplay.png"))

        self.stackedLayout = QtWidgets.QStackedLayout()
        self.stackedFrame = QtWidgets.QFrame()
        self.stackedFrame.setLayout(self.stackedLayout)

        self.mainLayout = QtWidgets.QGridLayout()
        self.mainLayout.setSpacing(0)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)

        self.storedPassword = self.config['password']
        self.addBasicTab()
        self.addReadinessTab()
        self.addSyncTab()
        self.addChatTab()
        self.addMessageTab()
        self.addMiscTab()
        self.tabList()

        if isMacOS():
            self.createMenubar()
            self.config['menuBar'] = dict()
            self.config['menuBar']['bar'] = self.menuBar
            self.config['menuBar']['editMenu'] = self.editMenu
        else:
            self.config['menuBar'] = None

        self.mainLayout.addWidget(self.stackedFrame, 0, 1)
        self.addBottomLayout()
        self.updatePasswordVisibilty()

        if self.getMoreState() == False:
            self.tabListFrame.hide()
            self.resetButton.hide()
            self.playerargsTextbox.hide()
            self.playerargsLabel.hide()
            self.runButton.hide()
            if self.mediapathTextbox.text() == "":
                self.mediapathTextbox.hide()
                self.mediapathLabel.hide()
                self.mediabrowseButton.hide()
            else:
                self.mediapathTextbox.show()
                self.mediapathLabel.show()
                self.mediabrowseButton.show()
            if isMacOS():
                newHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+50
            else:
                newHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+13
            if self.error:
                newHeight += self.errorLabel.height() + 3
            self.stackedFrame.setFixedHeight(newHeight)
        else:
            self.showmoreCheckbox.setChecked(True)
            self.tabListWidget.setCurrentRow(0)
            self.stackedFrame.setFixedHeight(self.stackedFrame.minimumSizeHint().height())


        self.showmoreCheckbox.toggled.connect(self.moreToggled)

        self.setLayout(self.mainLayout)
        if self.config['noStore']:
            self.runButton.setFocus()
        else:
            self.storeAndRunButton.setFocus()
        if isMacOS():
            initialHeight = self.connectionSettingsGroup.minimumSizeHint().height()+self.mediaplayerSettingsGroup.minimumSizeHint().height()+self.bottomButtonFrame.minimumSizeHint().height()+50
            if self.error:
                initialHeight += 40
            self.setFixedWidth(self.sizeHint().width())
            self.setFixedHeight(initialHeight)
        else:
            self.setFixedSize(self.sizeHint())
        self.setAcceptDrops(True)

        if constants.SHOW_TOOLTIPS:
            self.processWidget(self, lambda w: self.loadTooltips(w))
        self.processWidget(self, lambda w: self.loadValues(w))
        self.processWidget(self, lambda w: self.connectChildren(w))
        self.populateEmptyServerList()



# coding:utf8

"""Italian dictionary"""

# Filename, dictionary name and LANGUAGE-TAG value based on ISO country code. Language tag listed at https://learn.microsoft.com/en-us/openspecs/windows_protocols/ms-lcid/a9eac961-e77d-41a6-90a5-ce1a8b0cdb9c?redirectedfrom=MSDN

it = {
    "LANGUAGE": "Italiano",
    "LANGUAGE-TAG": "it",

    # Strings for Windows NSIS installer
    "installer-language-file": "Italian.nlf", # Relevant .nlf file at https://github.com/kichik/nsis/tree/master/Contrib/Language%20files
    "installer-associate": "Associa Syncplay con i file multimediali.",
    "installer-shortcut": "Crea i collegamenti nei percorsi seguenti:",
    "installer-start-menu": "Menu Start",
    "installer-desktop": "Desktop",
    "installer-quick-launch-bar": "Barra di avvio rapido",
    "installer-automatic-updates": "Controllo automatico degli aggiornamenti",
    "installer-uninstall-configuration": "Cancella i file di configurazione.",

    # Client notifications
    "config-cleared-notification": "Impostazioni iniziali ripristinate. I cambiamenti saranno memorizzati quando salverai una configurazione valida.",

    "relative-config-notification": "Caricato i file di configurazione relativi: {}",

    "connection-attempt-notification": "Tentativo di connessione a {}:{}",  # Port, IP
    "reconnection-attempt-notification": "Connessione col server persa, tentativo di riconnesione in corso",
    "disconnection-notification": "Disconnesso dal server",
    "connection-failed-notification": "Connessione col server fallita",
    "connected-successful-notification": "Connessione al server effettuata con successo",
    "retrying-notification": "%s, Nuovo tentativo in %d secondi...",  # Seconds
    "reachout-successful-notification": "Collegamento stabilito con {} ({})",

    "rewind-notification": "Riavvolgo a causa della differenza temporale con {}",  # User
    "fastforward-notification": "Avanzamento rapido a causa della differenza temporale con {}",  # User
    "slowdown-notification": "Rallento a causa della differenza temporale con {}",  # User
    "revert-notification": "Ripristino la velocità di riproduzione normale",

    "pause-notification": "{} ha messo in pausa ({})",  # User, Time - TODO: Change into format "{} paused at {}" in line with English message
    "unpause-notification": "{} ha ripreso la riproduzione",  # User
    "seek-notification": "{} è passato da {} a {}",  # User, from time, to time

    "current-offset-notification": "Offset corrente: {} secondi",  # Offset

    "media-directory-list-updated-notification": "Le cartelle multimediali di Syncplay sono state aggiornate.",

    "room-join-notification": "{} è entranto nella stanza: '{}'",  # User
    "left-notification": "{} ha lasciato la stanza",  # User
    "left-paused-notification": "{} ha lasciato la stanza, {} ha messo in pausa",  # User who left, User who paused
    "playing-notification": "{} sta riproducendo '{}' ({})",  # User, file, duration
    "playing-notification/room-addendum":  " nella stanza: '{}'",  # Room

    "not-all-ready": "Non pronti: {}",  # Usernames
    "all-users-ready": "Tutti i partecipanti sono pronti ({} utenti)",  # Number of ready users
    "ready-to-unpause-notification": "Ora sei pronto - premi ancora una volta per riprendere la riproduzione",
    "set-as-ready-notification": "Ora sei pronto",
    "set-as-not-ready-notification": "Non sei pronto",
    "autoplaying-notification": "Riproduzione automatica in {}...",  # Number of seconds until playback will start

    "identifying-as-controller-notification": "Ti sei identificato come gestore della stanza con password '{}'...",
    "failed-to-identify-as-controller-notification": "{} ha fallito l'identificazione come gestore della stanza.",
    "authenticated-as-controller-notification": "{} autenticato come gestore della stanza",
    "created-controlled-room-notification": "Stanza gestita '{}' creata con password '{}'. Per favore salva queste informazioni per una consultazione futura!\n\nIn managed rooms everyone is kept in sync with the room operator(s) who are the only ones who can pause, unpause, seek, and change the playlist.\n\nYou should ask regular viewers to join the room '{}' but the room operators can join the room '{}' to automatically authenticate themselves.", # RoomName, operatorPassword, roomName, roomName:operatorPassword # TODO: Translate

    "other-set-as-ready-notification": "{} was set as ready by {}", # User set as ready, user who set them as ready # TODO: Translate
    "other-set-as-not-ready-notification": "{} was set as not ready by {}", # User set as not ready, user who set them as not ready # TODO: Translate

    "file-different-notification": "Il file che stai riproducendo sembra essere diverso da quello di {}",  # User
    "file-differences-notification": "Il tuo file mostra le seguenti differenze: {}",  # Differences
    "room-file-differences": "Differenze: {}",  # File differences (filename, size, and/or duration)
    "file-difference-filename": "nome",
    "file-difference-filesize": "dimensione",
    "file-difference-duration": "durata",
    "alone-in-the-room": "Non ci sono altri utenti nella stanza",

    "different-filesize-notification": " (la dimensione del tuo file è diversa da quella degli altri partecipanti!)",
    "userlist-playing-notification": "{} sta riproducendo:",  # Username
    "file-played-by-notification": "File: {} è in riproduzione da:",  # File
    "no-file-played-notification": "{} non sta riproducendo alcun file",  # Username
    "notplaying-notification": "Partecipanti che non stanno riproducendo alcun file:",
    "userlist-room-notification":  "Nella stanza '{}':",  # Room
    "userlist-file-notification": "File",
    "controller-userlist-userflag": "Gestore",
    "ready-userlist-userflag": "Pronto",

    "update-check-failed-notification": "Controllo automatico degli aggiornamenti di Syncplay {} fallito. Vuoi visitare https://syncplay.pl/ per verificare manualmente la presenza di aggiornamenti?",  # Syncplay version
    "syncplay-uptodate-notification": "Syncplay è aggiornato",
    "syncplay-updateavailable-notification": "Una nuova versione di Syncplay è disponibile. Vuoi visitare la pagina delle release?",

    "mplayer-file-required-notification": "Utilizzare Syncplay con mplayer di selezionare il file all'avvio",
    "mplayer-file-required-notification/example": "Esempio di utilizzo: syncplay [opzioni] [url|percorso/]nomefile",
    "mplayer2-required": "Syncplay non è compatibile con MPlayer 1.x, per favore utilizza mplayer2 or mpv",

    "unrecognized-command-notification": "Comando non riconosciuto",
    "commandlist-notification": "Comandi disponibili:",
    "commandlist-notification/room": "\tr [nome] - cambia stanza",
    "commandlist-notification/list": "\tl - mostra la lista di utenti",
    "commandlist-notification/undo": "\tu - annulla l'ultima ricerca",
    "commandlist-notification/pause": "\tp - attiva o disattiva la pausa",
    "commandlist-notification/seek": "\t[s][+-]tempo - salta all'istante di tempo dato, se + o - non è specificato si considera il tempo assoluto in secondi o min:sec",
    "commandlist-notification/offset": "\to[+-]duration - offset local playback by the given duration (in seconds or min:sec) from the server seek position - this is a deprecated feature", # TODO: Translate
    "commandlist-notification/help": "\th - mostra questo help",
    "commandlist-notification/toggle": "\tt - attiva o disattiva la funzionalità \"pronto\"",
    "commandlist-notification/setready": "\tsr [name] - sets user as ready",  # TODO: Translate
    "commandlist-notification/setnotready": "\tsn [name] - sets user as not ready",  # TODO: Translate
    "commandlist-notification/create": "\tc [nome] - crea una stanza gestita usando il nome della stanza attuale",
    "commandlist-notification/auth": "\ta [password] - autentica come gestore della stanza, utilizzando la password del gestore",
    "commandlist-notification/chat": "\tch [message] - invia un messaggio nella chat della stanza",
    "commandList-notification/queue": "\tqa [file/url] - add file or url to bottom of playlist",  # TO DO: Translate
    "commandList-notification/queueandselect": "\tqas [file/url] - add file or url to bottom of playlist and select it",  # TO DO: Translate
    "commandList-notification/playlist": "\tql - show the current playlist",  # TO DO: Translate
    "commandList-notification/select": "\tqs [index] - select given entry in the playlist",  # TO DO: Translate
    "commandList-notification/next": "\tqn - select next entry in the playlist", # TODO: Translate
    "commandList-notification/delete": "\tqd [index] - delete the given entry from the playlist",  # TO DO: Translate
    "syncplay-version-notification": "Versione di Syncplay: {}",  # syncplay.version
    "more-info-notification": "Maggiori informazioni a: {}",  # projectURL

    "gui-data-cleared-notification": "Syncplay ha ripristinato i dati dell'interfaccia relativi ai percorsi e allo stato delle finestre.",
    "language-changed-msgbox-label": "La lingua sarà cambiata quando avvierai Syncplay.",
    "promptforupdate-label": "Ti piacerebbe che, di tanto in tanto, Syncplay controllasse automaticamente la presenza di aggiornamenti?",

    "media-player-latency-warning": "Attenzione: il media player ha impiegato {} secondi per rispondere. Se stai avendo problemi di sincronizzazione, chiudi delle applicazioni per liberare le risorse di sistema e, se ciò non dovesse avere alcun effetto, prova un altro media player.",  # Seconds to respond
    "mpv-unresponsive-error": "mpv non ha risposto per {} secondi, quindi sembra non funzionare correttamente. Per favore, riavvia Syncplay.",  # Seconds to respond

    # Client prompts
    "enter-to-exit-prompt": "Premi Invio per uscire\n",

    # Client errors
    "missing-arguments-error": "Alcuni argomenti obbligatori non sono stati trovati. Fai riferimento a --help",
    "server-timeout-error": "Connessione col server scaduta",
    "mpc-slave-error": "Non è possibile avviare MPC in modalità slave!",
    "mpc-version-insufficient-error": "La tua versione di MPC è troppo vecchia, per favore usa `mpc-hc` >= `{}`",
    "mpc-be-version-insufficient-error": "La tua versione di MPC è troppo vecchia, per favore usa `mpc-be` >= `{}`",
    "mpv-version-error": "Syncplay non è compatibile con questa versione di mpv. Per favore usa un'altra versione di mpv (es. Git HEAD).",
    "mpv-failed-advice": "The reason mpv cannot start may be due to the use of unsupported command line arguments or an unsupported version of mpv.", # TODO: Translate
    "player-file-open-error": "Il player non è riuscito ad aprire il file",
    "player-path-error": "Il path del player non è configurato correttamente. I player supportati sono: mpv, mpv.net, VLC, MPC-HC, MPC-BE, mplayer2 e IINA",
    "hostname-empty-error": "Il campo hostname non può essere vuoto",
    "empty-error": "Il campo {} non può esssere vuoto",  # Configuration
    "media-player-error": "Errore media player: \"{}\"",  # Error line
    "unable-import-gui-error": "Non è possibile importare le librerie di interfaccia grafica. Hai bisogno di PySide per poter utilizzare l'interfaccia grafica. If you want to run Syncplay in console mode then run it with the --no-gui command line switch. See https://syncplay.pl/guide/ for more details.", # TODO: Translate end of message and update second sentence to be a translation of "You need to have the correct version of PySide installed for the GUI to work."
    "unable-import-twisted-error": "Non è possibile importare Twisted. Si prega di installare Twisted v16.4.0 o superiore.",

    "arguments-missing-error": "Alcuni argomenti obbligatori non sono stati trovati. Fai riferimento a --help",

    "unable-to-start-client-error": "Impossibile avviare il client",

    "player-path-config-error": "Il percorso del player non è configurato correttamente. I player supportati sono: mpv, mpv.net, VLC, MPC-HC, MPC-BE, mplayer2 e IINA.",
    "no-file-path-config-error": "Deve essere selezionato un file prima di avviare il player",
    "no-hostname-config-error": "Il campo hostname non può essere vuoto",
    "invalid-port-config-error": "La porta deve essere valida",
    "empty-value-config-error": "Il campo {} non può essere vuoto",  # Config option

    "not-json-error": "Non è una stringa con codifica JSON\n",
    "hello-arguments-error": "Not enough Hello arguments\n",  # DO NOT TRANSLATE
    "version-mismatch-error": "La versione del client è diversa da quella del server\n",
    "vlc-failed-connection": "Impossibile collegarsi a VLC. Se non hai installato syncplay.lua e stai usando l'ultima versione di VLC, fai riferimento a https://syncplay.pl/LUA/ per istruzioni. Syncplay and VLC 4 are not currently compatible, so either use VLC 3 or an alternative such as mpv.", # TO DO: TRANSLATE
    "vlc-failed-noscript": "VLC ha segnalato che lo script di interfaccia syncplay.lua non è stato installato. Per favore, fai riferimento a https://syncplay.pl/LUA/ per istruzioni.",
    "vlc-failed-versioncheck": "Questa versione di VLC non è supportata da Syncplay.",
    "vlc-initial-warning": 'VLC does not always provide accurate position information to Syncplay, especially for .mp4 and .avi files. If you experience problems with erroneous seeking then please try an alternative media player such as <a href="https://mpv.io/">mpv</a> (or <a href="https://github.com/stax76/mpv.net/">mpv.net</a> for Windows users).', # TODO: Translate

    "feature-sharedPlaylists": "playlist condivise",  # used for not-supported-by-server-error
    "feature-chat": "chat",  # used for not-supported-by-server-error
    "feature-readiness": "pronto",  # used for not-supported-by-server-error
    "feature-managedRooms": "stanze gestite",  # used for not-supported-by-server-error
    "feature-setOthersReadiness": "readiness override",  # used for not-supported-by-server-error # TODO: Translate

    "not-supported-by-server-error": "La feature {} non è supportata da questo server..",  # feature
    "shared-playlists-not-supported-by-server-error": "Le playlist condivise potrebbero non essere supportata dal server. È necessario un server con Syncplay {}+ per assicurarsi che funzionino correttamente, tuttavia il server sta utilizzando Syncplay {}.",  # minVersion, serverVersion
    "shared-playlists-disabled-by-server-error": "Le playlist condivise sono state disabilitate nella configurazione del server. Per utilizzarle, dovrai collegarti a un altro server.",

    "invalid-seek-value": "Valore di ricerca non valido",
    "invalid-offset-value": "Valore di offset non valido",

    "switch-file-not-found-error": "Impossibile selezionare il file '{0}'. Syncplay osserva solo le cartelle multimediali specificate.",  # File not found
    "folder-search-timeout-error": "La ricerca nelle cartelle multimediali è stata interrotta perché l'analisi di '{}' sta impiegando troppo tempo (after having processed the first {:,} files). Ciò accade se si aggiunge nella lista di ricerca una cartella con troppe sottocartelle. Per riabilitare la selezione automatica dei file seleziona File->Imposta cartelle multimediali nella barra dei menù e rimuovi questa cartella, o sostituiscila con una sottocartella appropriata. Se la cartella è idonea, è possibile riabilitarla selezionando File->Imposta cartelle multimediali e premendo 'OK'.", # Folder, Files processed - Note: {:,} is {} but with added commas seprators - TODO: Translate
    "folder-search-timeout-warning": "Warning: It has taken {} seconds to scan {:,} files in the folder '{}'. This will occur if you select a folder with too many sub-folders in your list of media folders to search through or if there are too many files to process.",  # Folder, Files processed. Note: {:,} is {} but with added commas seprators. TODO: Translate
    "folder-search-first-file-timeout-error": "La ricerca dei media in '{}' è stata interrotta perché l'accesso alla cartella sta impiegando troppo tempo. Ciò accade se questa si trova in un disco di rete oppure se hai impostato il blocco della rotazione del disco rigido dopo un certo periodo di inattività. Per riabilitare la selezione automatica dei file seleziona File->Imposta cartelle multimediali, quindi rimuovi la cartella oppure risolvi il problema (es. cambiando le impostazioni di risparmio energetico).",  # Folder
    "added-file-not-in-media-directory-error": "Hai selezionato un file in '{}', che non è impostata come cartella multimediale. Puoi aggiungerla come cartella multimediale selezionando File->Imposta cartelle multimediali nella barra dei menù.",  # Folder
    "no-media-directories-error": "Nessuna cartella multimediale è stata configurata. Per permettere il corretto funzionamento delle playlist condivise e la selezione automatica dei file, naviga in File->Imposta cartelle multimediali e specifica dove Syncplay deve ricercare i file multimediali.",
    "cannot-find-directory-error": "Impossibile trovare la cartella multimediale '{}'. Per aggiornare la lista delle cartelle multimediali seleziona File->Imposta cartelle multimediali dalla barra dei menù e specifica dove Syncplay deve ricercare i file multimediali.",

    "failed-to-load-server-list-error": "Impossibile caricare la lista dei server pubblici. Per favore, visita https://www.syncplay.pl/ con il tuo browser.",

    # Client arguments
    "argument-description": 'Programma per sincronizzare la riproduzione di media player multipli attraverso la rete.',
    "argument-epilog": 'Se non è specificata alcuna opzione saranno utilizzati i valori _config',
    "nogui-argument": 'non mostrare l\'interfaccia grafica',
    "host-argument": 'indirizzo del server',
    "name-argument": 'username desiderato',
    "debug-argument": 'modalità debug',
    "force-gui-prompt-argument": 'mostra la finestra di configurazione',
    "no-store-argument": 'non salvare i valori in .syncplay',
    "room-argument": 'stanza di default',
    "password-argument": 'password del server',
    "player-path-argument": 'percorso dell\'eseguibile del tuo player',
    "file-argument": 'file da riprodurre',
    "args-argument": 'opzioni del player, se hai bisogno di utilizzare opzioni che iniziano con - anteponi un singolo \'--\'',
    "clear-gui-data-argument": 'ripristina il percorso e i dati impostati tramite interfaccia grafica e salvati come QSettings',
    "language-argument": 'lingua per i messaggi di Syncplay ({})', # Languages

    "version-argument": 'mostra la tua versione',
    "version-message": "Stai usando la versione di Syncplay {} ({})",

    "load-playlist-from-file-argument": "loads playlist from text file (one entry per line)", # TODO: Translate

    # Client labels
    "config-window-title": "Configurazione di Syncplay",

    "connection-group-title": "Impostazioni di connessione",
    "host-label": "Indirizzo del server: ",
    "name-label": "Username (opzionale):",
    "password-label": "Password del server (se necessaria):",
    "room-label": "Stanza di default: ",
    "roomlist-msgbox-label": "Edit room list (one per line)", # TODO: Translate

    "media-setting-title": "Impostazioni del media player",
    "executable-path-label": "Percorso del media player:",
    "media-path-label": "Percorso del video (opzionale):",
    "player-arguments-label": "Opzioni del player (se necessarie):",
    "browse-label": "Sfoglia",
    "update-server-list-label": "Aggiorna lista",

    "more-title": "Mostra altre impostazioni",
    "never-rewind-value": "Mai",
    "seconds-suffix": " sec",
    "privacy-sendraw-option": "Invio semplice",
    "privacy-sendhashed-option": "Invio cifrato",
    "privacy-dontsend-option": "Non inviare",
    "filename-privacy-label": "Nome del file:",
    "filesize-privacy-label": "Dimensione del file:",
    "checkforupdatesautomatically-label": "Controlla automaticamente gli aggiornamenti di Syncplay",
    "autosavejoinstolist-label": "Add rooms you join to the room list", # TO DO: Translate
    "slowondesync-label": "Rallenta in caso di sfasamento minimo (non supportato su MPC-HC/BE)",
    "rewindondesync-label": "Riavvolgi in caso di grande sfasamento (consigliato)",
    "fastforwardondesync-label": "Avanzamento rapido in caso di ritardo (consigliato)",
    "dontslowdownwithme-label": "Non rallentare o riavvolgere gli altri utenti (sperimentale)",
    "pausing-title": "Pausa",
    "pauseonleave-label": "Metti in pausa quando gli altri utenti lasciano la stanza (es. disconnessione)",
    "readiness-title": "Stato iniziale di 'pronto'",
    "readyatstart-label": "Imposta sempre il mio stato come \"pronto\" a guardare",
    "forceguiprompt-label": "Non mostrare la finestra di configurazione di Syncplay a ogni avvio",  # (Inverted)
    "showosd-label": "Abilita i messaggi OSD",

    "showosdwarnings-label": "Mostra gli avvisi (es. file differenti, utenti non pronti)",
    "showsameroomosd-label": "Mostra gli eventi della tua stanza",
    "shownoncontrollerosd-label": "Mostra gli eventi dei non gestori nelle stanze gestite",
    "showdifferentroomosd-label": "Mostra gli eventi di altre stanze",
    "showslowdownosd-label": "Mostra le notifiche di rallentamento / riavvolgimento",
    "language-label": "Lingua:",
    "automatic-language": "Predefinita ({})",  # Default language
    "showdurationnotification-label": "Avvisa in caso di mancata corrispondenza della durata del file",
    "basics-label": "Generali",
    "readiness-label": "Play/Pausa",
    "misc-label": "Varie",
    "core-behaviour-title": "Comportamento principale della stanza",
    "syncplay-internals-title": "Funzionamento di Syncplay",
    "syncplay-mediasearchdirectories-title": "Cartelle contenenti i file multimediali",
    "syncplay-mediasearchdirectories-label": "Cartelle contenenti i file multimediali (un solo percorso per riga)",
    "sync-label": "Sincronia",  # don't have better options as the label won't fit in the panel.
    "sync-otherslagging-title": "Se gli altri partecipanti non sono sincronizzati...",
    "sync-youlaggging-title": "Se tu sei non sei sincronizzato...",
    "messages-label": "Messaggi",
    "messages-osd-title": "Impostazioni On-Screen Display",
    "messages-other-title": "Altre impostazioni dello schermo",
    "chat-label": "Chat",
    "privacy-label": "Privacy",  # Currently unused, but will be brought back if more space is needed in Misc tab
    "privacy-title": "Impostazioni privacy",
    "unpause-title": "Premendo play, imposta il tuo stato su \"pronto\" e:",
    "unpause-ifalreadyready-option": "Riprendi la riproduzione se eri già pronto",
    "unpause-ifothersready-option": "Riprendi la riproduzione se eri già pronto o se gli altri partecipanti sono pronti (default)",
    "unpause-ifminusersready-option": "Riprendi la riproduzione se eri già pronto o se un numero minimo di partecipanti è pronto",
    "unpause-always": "Riprendi sempre la riproduzione",
    "syncplay-trusteddomains-title": "Domini fidati (per streaming e i contenuti in rete)",

    "chat-title": "Inserimento messaggi di chat",
    "chatinputenabled-label": "Abilita la chat su mpv",
    "chatdirectinput-label": "Abilita la chat istantanea (evita di dover premere Invio per chattare)",
    "chatinputfont-label": "Font dell'input della chat",
    "chatfont-label": "Imposta font",
    "chatcolour-label": "Imposta colore",
    "chatinputposition-label": "Posizione dell'area di inserimento testo in mpv",
    "chat-top-option": "In alto",
    "chat-middle-option": "Al centro",
    "chat-bottom-option": "In basso",
    "chatoutputheader-label": "Output messaggi di chat",
    "chatoutputfont-label": "Font dell'output della chat",
    "chatoutputenabled-label": "Abilita l'output della chat nel media player (al momento solo mpv è supportato)",
    "chatoutputposition-label": "Modalità di output",
    "chat-chatroom-option": "Stile chatroom",
    "chat-scrolling-option": "A scorrimento",

    "mpv-key-tab-hint": "[TAB] per attivare le scorciatoie da tastiera e disattivare la chat.",
    "mpv-key-hint": "[Invio] per inviare un messaggio. [Esc] per uscire dalla modalità chat.",
    "alphakey-mode-warning-first-line": "Puoi utilizzare temporaneamente i vecchi comandi di mpv con i tasti a-z.",
    "alphakey-mode-warning-second-line": "Premi [TAB] per ritornare alla modalità chat di Syncplay.",

    "help-label": "Aiuto",
    "reset-label": "Elimina configurazione",
    "run-label": "Avvia Syncplay",
    "storeandrun-label": "Salva la configurazione e avvia Syncplay",

    "contact-label": "Sentiti libero di inviare un'e-mail a <a href=\"mailto:dev@syncplay.pl\"><nobr>dev@syncplay.pl</nobr></a>, <a href=\"https://github.com/Syncplay/syncplay/issues\"><nobr>segnalare un problema</nobr></a> su GitHub / <a href=\"https://github.com/Syncplay/syncplay/discussions\"><nobr>start a discussion</nobr></a> to make a suggestion or ask a question via GitHub,, <a href=\"https://www.facebook.com/SyncplaySoftware\"><nobr>lasciare un like sulla nostra pagina Facebook</nobr></a>, <a href=\"https://twitter.com/Syncplay/\"><nobr>seguirci su Twitter</nobr></a>, o visitare <a href=\"https://syncplay.pl/\"><nobr>https://syncplay.pl/</nobr></a>. Non usare Syncplay per inviare dati sensibili.", # TODO: Update translation

    "joinroom-label": "Entra nella stanza",
    "joinroom-menu-label": "Entra nella stanza {}",
    "seektime-menu-label": "Vai a...",
    "undoseek-menu-label": "Annulla vai a...",
    "play-menu-label": "Play",
    "pause-menu-label": "Pausa",
    "playbackbuttons-menu-label": "Mostra i controlli della riproduzione",
    "autoplay-menu-label": "Mostra il tasto di riproduzione automatica",
    "autoplay-guipushbuttonlabel": "Riproduci quando tutti sono pronti",
    "autoplay-minimum-label": "Minimo utenti pronti:",
    "hideemptyrooms-menu-label": "Hide empty persistent rooms", # TODO: Translate

    "sendmessage-label": "Invia",

    "ready-guipushbuttonlabel": "Sono pronto a guardare!",

    "roomuser-heading-label": "Stanza / Utente",
    "size-heading-label": "Dimensione",
    "duration-heading-label": "Durata",
    "filename-heading-label": "Nome del file",
    "notifications-heading-label": "Notifiche",
    "userlist-heading-label": "Lista degli utenti nella stanza",

    "browseformedia-label": "Seleziona i file multimediali",

    "file-menu-label": "&File",  # & precedes shortcut key
    "openmedia-menu-label": "&Apri file multimediali",
    "openstreamurl-menu-label": "Apri indirizzo di &rete",
    "setmediadirectories-menu-label": "Imposta &cartelle multimediali",
    "loadplaylistfromfile-menu-label": "&Load playlist from file",  # TODO: Translate
    "saveplaylisttofile-menu-label": "&Save playlist to file",  # TODO: Translate
    "exit-menu-label": "&Esci",
    "advanced-menu-label": "&Avanzate",
    "window-menu-label": "&Finestra",
    "setoffset-menu-label": "Imposta &offset",
    "createcontrolledroom-menu-label": "&Crea stanza gestita",
    "identifyascontroller-menu-label": "&Identificati come operatore della stanza",
    "settrusteddomains-menu-label": "Imposta &domini fidati",
    "addtrusteddomain-menu-label": "Aggiungi {} come dominio fidato",  # Domain

    "edit-menu-label": "&Modifica",
    "cut-menu-label": "&Taglia",
    "copy-menu-label": "&Copia",
    "paste-menu-label": "&Incolla",
    "selectall-menu-label": "&Seleziona tutto",

    "playback-menu-label": "&Riproduzione",

    "help-menu-label": "&Aiuto",
    "userguide-menu-label": "Apri guida &utente",
    "update-menu-label": "Controlla la presenza di &aggiornamenti",

    "startTLS-initiated": "Tentativo di connessione sicura in corso",
    "startTLS-secure-connection-ok": "Connessione sicura stabilita ({})",
    "startTLS-server-certificate-invalid": 'Connessione sicura non riuscita. Il certificato di sicurezza di questo server non è valido. La comunicazione potrebbe essere intercettata da una terza parte. Per ulteriori dettagli e informazioni sulla risoluzione del problema, clicca <a href="https://syncplay.pl/trouble">qui</a>.',
    "startTLS-server-certificate-invalid-DNS-ID": "Syncplay does not trust this server because it uses a certificate that is not valid for its hostname.", # TODO: Translate
    "startTLS-not-supported-client": "Questo client non supporta TLS",
    "startTLS-not-supported-server": "Questo server non supporta TLS",

    # TLS certificate dialog
    "tls-information-title": "Informazioni sul certificato",
    "tls-dialog-status-label": "<strong>Syncplay è connesso a {} tramite una connessione codificata.</strong>",
    "tls-dialog-desc-label": "La codifica con un certificato digitale mantiene private le informazioni quando vengono<br/>inviate dal/al server {}.",
    "tls-dialog-connection-label": "Informazioni codificate usando Transport Layer Security (TLS), versione {} usando gli<br/>algoritmi di cifratura: {}.",
    "tls-dialog-certificate-label": "Certificato rilasciato da {} valido fino al {}.",

    # About dialog
    "about-menu-label": "&Informazioni su Syncplay",
    "about-dialog-title": "Informazioni su Syncplay",
    "about-dialog-release": "Versione {} release {}",
    "about-dialog-license-text": "Rilasciato sotto Apache&nbsp;License,&nbsp;Version 2.0",
    "about-dialog-license-button": "Licenza",
    "about-dialog-dependencies": "Dipendenze",

    "setoffset-msgbox-label": "Imposta offset",
    "offsetinfo-msgbox-label": "Offset (vedi https://syncplay.pl/guide/ per istruzioni):",

    "promptforstreamurl-msgbox-label": "Apri URL",
    "promptforstreamurlinfo-msgbox-label": "Indirizzo di rete",

    "addfolder-label": "Aggiungi cartella",

    "adduris-msgbox-label": "Aggiungi gli indirizzi alla playlist (uno per riga)",
    "editplaylist-msgbox-label": "Imposta playlist (una per riga)",
    "trusteddomains-msgbox-label": "Domini a cui è lecito passare automaticamente (uno per riga)",

    "createcontrolledroom-msgbox-label": "Crea stanza gestita",
    "controlledroominfo-msgbox-label": "Inserisci il nome della stanza gestita\r\n(vedi https://syncplay.pl/guide/ per istruzioni):",

    "identifyascontroller-msgbox-label": "Identificati come operatore della stanza",
    "identifyinfo-msgbox-label": "Inserisci la password dell'operatore per questa stanza\r\n(vedi https://syncplay.pl/guide/ per istruzioni):",

    "public-server-msgbox-label": "Seleziona il server pubblico per questa sessione",

    "megabyte-suffix": " MB",

    # Tooltips

    "host-tooltip": "Hostname o indirizzo IP a cui collegarsi e, se necessario, includere la porta (es. syncplay.pl:8999). La sincronizzazione avviene solo con gli utenti collegati allo stesso server/porta.",
    "name-tooltip": "Il nome utente con cui sarai riconosciuto. Nessuna registrazione necessaria, cosi potrai sempre cambiarlo. Se lasciato vuoto, viene scelto un nome casuale.",
    "password-tooltip": "La password è necessaria solo in caso di connessione a server privati.",
    "room-tooltip": "La stanza in cui entrare dopo la connessione. Può assumere qualsiasi nome, ma ricorda che sarai sincronizzato solo con gli utenti nella stessa stanza.",

    "edit-rooms-tooltip": "Edit room list.",  # TO DO: Translate

    "executable-path-tooltip": "Percorso del media player desiderato (scegliere tra mpv, mpv.net, VLC, MPC-HC/BE, mplayer2 o IINA).",
    "media-path-tooltip": "Percorso del video o stream da aprire. Necessario per mplayer2.",
    "player-arguments-tooltip": "Argomenti da linea di comando aggiuntivi da passare al media player scelto.",
    "mediasearcdirectories-arguments-tooltip": "Cartelle dove Syncplay cercherà i file multimediali, es. quando usi la funzione click to switch. Syncplay cercherà anche nelle sottocartelle.",

    "more-tooltip": "Mostra le impostazioni usate meno frequentemente.",
    "filename-privacy-tooltip": "Modalità di invio al server del nome del file attualmente in riproduzione.",
    "filesize-privacy-tooltip": "Modalità di invio al server della dimensione del file attualmente in riproduzione.",
    "privacy-sendraw-tooltip": "Invia questa informazione in chiaro. Questa è l'impostazione predefinita per la maggior parte delle funzionalità.",
    "privacy-sendhashed-tooltip": "Invia una versione cifrata dell'informazione, rendendola meno visibile agli altri client.",
    "privacy-dontsend-tooltip": "Non inviare questa informazione al server. Questo garantisce massima privacy.",
    "checkforupdatesautomatically-tooltip": "Controlla regolarmente la presenza di nuove versioni di Syncplay.",
    "autosavejoinstolist-tooltip": "When you join a room in a server, automatically remember the room name in the list of rooms to join.", # TO DO: Translate
    "slowondesync-tooltip": "Riduce temporaneamente la velocità di riproduzione quando c'è bisogno di sincronizzarti con gli altri utenti. Non supportato su MPC-HC/BE.",
    "dontslowdownwithme-tooltip": "Gli altri utenti non vengono rallentati se non sei sincronizzato. Utile per i gestori della stanza.",
    "pauseonleave-tooltip": "Mette in pausa la riproduzione se vieni disconnesso o se qualcuno lascia la stanza.",
    "readyatstart-tooltip": "Imposta il tuo stato su \"pronto\" all'avvio (in caso contrario, sarai su \"non pronto\" finché non cambierai il tuo stato)",
    "forceguiprompt-tooltip": "La finestra di configurazione non viene mostrata quando apri Syncplay.",
    "nostore-tooltip": "Avvia Syncplay con la configurazione scelta, ma non salva le impostazioni.",
    "rewindondesync-tooltip": "Torna indietro quando necessario per ristabilire la sincronizzazione. Disabilitare quest'opzione può causare gravi problemi di sincronizzazione!",
    "fastforwardondesync-tooltip": "Avanza rapidamente quando non sei sincronizzato col gestore della stanza (usa una posizione fittizia se 'Non rallentare o riavvolgere gli altri utenti' è abilitato).",
    "showosd-tooltip": "Invia i messaggi di Syncplay al media player tramite OSD.",
    "showosdwarnings-tooltip": "Mostra gli avvisi in caso di riproduzione di un file differente, se sei l'unico utente nella stanza, se ci sono utenti non pronti, ecc.",
    "showsameroomosd-tooltip": "Mostra le notifiche OSD per gli eventi relativi alla stanza in cui si trova l'utente.",
    "shownoncontrollerosd-tooltip": "Mostra le notifiche OSD per gli eventi relativi ai non operatori presenti nelle stanze gestite.",
    "showdifferentroomosd-tooltip": "Mostra le notifiche OSD per gli eventi relativi alle stanze in cui l'utente non si trova.",
    "showslowdownosd-tooltip": "Mostra le notifiche di rallentamento / riavvolgimento in caso di differenza temporale.",
    "showdurationnotification-tooltip": "Utile quando manca un segmento di un file con più parti. Può causare dei falsi positivi.",
    "language-tooltip": "Lingua da utilizzare in Syncplay.",
    "unpause-always-tooltip": "Se riprendi la riproduzione, il tuo stato cambia in \"pronto\" e la riproduzione viene avviata, piuttosto che impostarti solo su pronto.",
    "unpause-ifalreadyready-tooltip": "Se riprendi la riproduzione quando non sei \"pronto\", verrai impostato su pronto - ripeti il comando ancora una volta per avviare la riproduzione.",
    "unpause-ifothersready-tooltip": "Se riprendi la riproduzione quando non sei \"pronto\" la riproduzione verrà avviata solo se gli altri sono pronti.",
    "unpause-ifminusersready-tooltip": "Se riprendi la riproduzione quando non sei \"pronto\", la riproduzione verrà avviata solo se un numero minimo di utenti è \"pronto\".",
    "trusteddomains-arguments-tooltip": "Domini verso cui è possibile collegarsi automaticamente quando le playlist condivise sono abilitate.",

    "chatinputenabled-tooltip": "Abilita l'input della chat in mpv (premi Invio per chattare, per inviare ed Esc per cancellare)",
    "chatdirectinput-tooltip": "Evita di dover premere Invio per aprire l'input della chat in mpv. Premi TAB in mpv per disabilitare temporaneamente questa funzione.",
    "font-label-tooltip": "Font usato nell'input della chat in mpv. Non influenza cosa vedono gli altri, vale solo per te.",
    "set-input-font-tooltip": "Font usato nell'input della chat in mpv. Non influenza cosa vedono gli altri, vale solo per te.",
    "set-input-colour-tooltip": "Colore del font usato nell'input della chat in mpv. Non influenza cosa vedono gli altri, vale solo per te.",
    "chatinputposition-tooltip": "Posizione dell'input della chat in mpv quando premi Invio.",
    "chatinputposition-top-tooltip": "Posiziona l'input della chat in cima alla finestra di mpv.",
    "chatinputposition-middle-tooltip": "Posizione l'input della chat al centro della finestra di mpv.",
    "chatinputposition-bottom-tooltip": "Posiziona l'input della chat in basso alla finestra di mpv.",
    "chatoutputenabled-tooltip": "Mostra i messaggi di chat nell'OSD (se supportato dal media player).",
    "font-output-label-tooltip": "Font dell'output della chat.",
    "set-output-font-tooltip": "Font usato per mostrare i messaggi di chat.",
    "chatoutputmode-tooltip": "Come sono mostrati i messaggi di chat.",
    "chatoutputmode-chatroom-tooltip": "Mostra i nuovi messaggi di chat al di sotto di quelli precedenti.",
    "chatoutputmode-scrolling-tooltip": "Scorri il testo della chat da destra a sinistra.",

    "help-tooltip": "Apri la guida utente su syncplay.pl.",
    "reset-tooltip": "Ripristina le impostazioni iniziali di Syncplay.",
    "update-server-list-tooltip": "Scarica la lista dei server pubblici da syncplay.pl.",

    "sslconnection-tooltip": "Connessione sicura al server. Clicca per informazioni sul certificato.",

    "joinroom-tooltip": "Lascia la stanza attuale e entra in quella specificata.",
    "seektime-msgbox-label": "Salta all'istante di tempo specificato (in secondi / min:sec). Usa +/- per una ricerca relativa.",
    "ready-tooltip": "Indica quando sei pronto a guardare.",
    "autoplay-tooltip": "Avvia la riproduzione automatica quando il numero minimo di utenti è pronto.",
    "switch-to-file-tooltip": "Doppio click per passare a {}",  # Filename
    "sendmessage-tooltip": "Invia il messaggio alla stanza",

    # In-userlist notes (GUI)
    "differentsize-note": "Dimensione file diversa!",
    "differentsizeandduration-note": "Durata e dimensione file diversi!",
    "differentduration-note": "Durata diversa!",
    "nofile-note": "(Nessun file in riproduzione)",

    # Server messages to client
    "new-syncplay-available-motd-message": "Stai usando Syncplay {} ma una nuova versione è disponibile presso https://syncplay.pl",  # ClientVersion
    "persistent-rooms-notice": "NOTICE: This server uses persistent rooms, which means that the playlist information is stored between playback sessions. If you want to create a room where information is not saved then put -temp at the end of the room name.", # TO DO: Translate - NOTE: Do not translate the word -temp to create a room where information is not saved then put -temp at the end of the room name.", # TO DO: Translate
    "ready-chat-message": "I have set {} as ready.",  # User # TODO: Translate
    "not-ready-chat-message": "I have set {} as not ready.",  # User # TODO: Translate

    # Server notifications
    "welcome-server-notification": "Benvenuto nel server Syncplay, ver. {0}",  # version
    "client-connected-room-server-notification": "{0}({2}) connesso alla stanza '{1}'",  # username, host, room
    "client-left-server-notification": "{0} ha lasciato il server",  # name
    "no-salt-notification": "NOTA BENE: In futuro, per consentire il corretto funzionamento delle password generate da questo server (per le stanze gestite), aggiungi da linea di comando il seguente argomento prima di avviare il server Syncplay: --salt {}",  # Salt


    # Server arguments
    "server-argument-description": 'Programma per sincronizzare la riproduzione di media player multipli attraverso la rete. Modulo server.',
    "server-argument-epilog": 'Se non è specificata alcuna opzione saranno utilizzati i valori _config',
    "server-port-argument": 'Porta TCP del server',
    "server-password-argument": 'password del server',
    "server-isolate-room-argument": 'Mantiene le stanze isolate',
    "server-salt-argument": "usare stringhe casuali per generare le password delle stanze gestite",
    "server-disable-ready-argument": "disabilita la funzionalità \"pronto\"",
    "server-motd-argument": "percorso del file da cui verrà letto il messaggio del giorno",
    "server-rooms-argument": "path to database file to use and/or create to store persistent room data. Enables rooms to persist without watchers and through restarts", # TODO: Translate
    "server-permanent-rooms-argument": "path to file which lists permenant rooms that will be listed even if the room is empty (in the form of a text file which lists one room per line) - requires persistent rooms to be enabled", # TODO: Translate
    "server-chat-argument": "abilita o disabilita la chat",
    "server-chat-maxchars-argument": "Numero massimo di caratteri in un messaggio di chat (default è {})", # Default number of characters
    "server-maxusernamelength-argument": "Numero massimo di caratteri in un nome utente (default è {})",
    "server-stats-db-file-argument": "Abilita la raccolta dei dati statistici nel file SQLite indicato",
    "server-startTLS-argument": "Abilita il protocollo TLS usando i certificati contenuti nel percorso indicato",
    "server-messed-up-motd-unescaped-placeholders": "Il messaggio del giorno ha dei caratteri non 'escaped'. Tutti i simboli $ devono essere doppi ($$).",
    "server-messed-up-motd-too-long": "Il messaggio del giorno è troppo lungo - numero massimo di caratteri è {}, {} trovati.",
    "server-listen-only-on-ipv4": "Listen only on IPv4 when starting the server.",
    "server-listen-only-on-ipv6": "Listen only on IPv6 when starting the server.",
    "server-interface-ipv4": "The IP address to bind to for IPv4. Leaving it empty defaults to using all.",
    "server-interface-ipv6": "The IP address to bind to for IPv6. Leaving it empty defaults to using all.",

    # Server errors
    "unknown-command-server-error": "Comando non riconosciuto {}",  # message
    "not-json-server-error": "Non è una stringa in codifica JSON {}",  # message
    "line-decode-server-error": "Non è una stringa utf-8",
    "not-known-server-error": "Devi essere autenticato dal server prima di poter inviare questo comando",
    "client-drop-server-error": "Il client è caduto: {} -- {}",  # host, error
    "password-required-server-error": "È richiesta una password",
    "wrong-password-server-error": "La password inserita è errata",
    "hello-server-error": "Not enough Hello arguments",  # DO NOT TRANSLATE

    # Playlists
    "playlist-selection-changed-notification": "{} ha cambiato il file selezionato nella playlist",  # Username
    "playlist-contents-changed-notification": "{} ha aggiornato la playlist",  # Username
    "cannot-find-file-for-playlist-switch-error": "Impossibile trovare il file {} nelle cartelle multimediali per permettere il cambio di file tramite la playlist!",  # Filename
    "cannot-add-duplicate-error": "Impossibile aggiungere una seconda voce per '{}' alla playlist. Non è possibile avere file duplicati.",  # Filename
    "cannot-add-unsafe-path-error": "Impossibile caricare automaticamente {} perché non è presente nei domini fidati. Puoi passare all'inserimento manuale facendo doppio click sull'indirizzo nella playlist, oppure aggiungerlo ai domini fidati tramite File->Avanzate->Imposta domini fidati. Cliccando col tasto destro del mouse su un indirizzo puoi impostare il suo dominio come fidato tramite il menù contestuale.",  # Filename
    "sharedplaylistenabled-label": "Abilita le playlist condivise",
    "removefromplaylist-menu-label": "Rimuovi dalla playlist",
    "shuffleremainingplaylist-menu-label": "Mescola i file non ancora riprodotti",
    "shuffleentireplaylist-menu-label": "Mescola l'intera playlist",
    "undoplaylist-menu-label": "Annulla l'ultima modifica alla playlist",
    "addfilestoplaylist-menu-label": "Aggiungi un file alla fine della playlist",
    "addurlstoplaylist-menu-label": "Aggiungi un indirizzo alla fine della playlist",
    "editplaylist-menu-label": "Modifica la playlist",

    "open-containing-folder": "Apri la cartella contenente questo file",
    "addyourfiletoplaylist-menu-label": "Aggiungi il tuo file alla playlist",
    "addotherusersfiletoplaylist-menu-label": "Aggiungi il file di {} alla playlist", # Username
    "addyourstreamstoplaylist-menu-label": "Aggiungi il tuo indirizzo alla playlist",
    "addotherusersstreamstoplaylist-menu-label": "Aggiungi l'indirizzo di {} alla playlist", # Username  # item owner indicator
    "openusersstream-menu-label": "Apri l'indirizzo di {}",  # [username]
    "openusersfile-menu-label": "Apri il file di {}",  # [username]'s

    "setasready-menu-label": "Set {} as ready",  # [Username] # TODO: Translate
    "setasnotready-menu-label": "Set {} as not ready",  # [Username] # TODO: Translate

    "playlist-instruction-item-message": "Trascina qui i file per aggiungerli alla playlist condivisa.",
    "sharedplaylistenabled-tooltip": "Gli operatori della stanza possono aggiungere i file a una playlist sincronizzata per garantire che tutti i partecipanti stiano guardando la stessa cosa. Configura le cartelle multimediali alla voce 'Miscellanea'.",

    "playlist-empty-error": "Playlist is currently empty.",  # TO DO: Translate
    "playlist-invalid-index-error": "Invalid playlist index", # TO DO: Translate
}
